<?php
if (!function_exists('stgh_wc_updater_get_plugin_data')) {
    function stgh_wc_updater_get_plugin_data($pluginFile, $readmeFile = false)
    {
        if (!function_exists('get_plugin_data')) {
            require_once(ABSPATH . 'wp-admin/includes/plugin.php');
        }

        $pluginData = get_plugin_data($pluginFile);
        if($readmeFile === false)
            $pluginData['sections'] = "";
        else
            $pluginData['sections'] = stgh_wc_updater_parse_readme($readmeFile);


        return $pluginData;

    }
}

if (!function_exists('stgh_wc_updater_improve_readme')) {
    function stgh_wc_updater_improve_readme($string)
    {
        $string = nl2br(esc_html($string));
        $string = preg_replace('/`(.*?)`/', '<code>\\1</code>', $string);
        $string = preg_replace('/[\040]\*\*(.*?)\*\*/', ' <strong>\\1</strong>', $string);
        $string = preg_replace('/[\040]\*(.*?)\*/', ' <em>\\1</em>', $string);
        $string = preg_replace('/= (.*?) =/', '<h4>\\1</h4>', $string);
        $string = preg_replace('/\*(.*?)/', '<li>\\1', $string);
        $string = preg_replace('/<\/h4>(<br \/>\n<br \/>)*/s', '</h4>', $string);
        $string = preg_replace('/<br \/>\n<h4>/s', '<h4>', $string);

        $string = preg_replace('/\[(.*)\]\((.*)\)/s', '<a href="\\2">\\1</a>', $string);

        return $string;
    }
}


if (!function_exists('stgh_wc_updater_parse_readme')) {

    function stgh_wc_updater_parse_readme($filePath)
    {

        $source = file_get_contents($filePath);

        if (!$source) {
            return false;
        }

        $sections = array();
        $sectionTags = array(
            'description' => '== Description ==',
            'screenshots' => '== Screenshots ==',
            'translation' => '== Translations ==',
            'installation' => '== Installation ==',
            'FAQ' => '== Frequently Asked Questions ==',
            'changelog' => '== Changelog ==');


        $syntaxOk = preg_match('/(' . PHP_EOL . ')*' . $sectionTags['description'] . '(' . PHP_EOL . ')*(.+)(' . PHP_EOL . ')*' . $sectionTags['screenshots'] . '/s', $source, $matches);
        if (!$syntaxOk) {
            return false;
        }
        $sections['description'] = stgh_wc_updater_improve_readme($matches[3]);

        $syntaxOk = preg_match('/(' . PHP_EOL . ')*' . $sectionTags['translation'] . '(' . PHP_EOL . ')*(.+)(' . PHP_EOL . ')*' . $sectionTags['installation'] . '/s', $source, $matches);
        if (!$syntaxOk) {
            return false;
        }
        $sections['translation'] = stgh_wc_updater_improve_readme($matches[3]);

        $syntaxOk = preg_match('/(' . PHP_EOL . ')*' . $sectionTags['installation'] . '(' . PHP_EOL . ')*(.+)(' . PHP_EOL . ')*' . $sectionTags['FAQ'] . '/s', $source, $matches);
        if (!$syntaxOk) {
            return false;
        }
        $sections['installation'] = stgh_wc_updater_improve_readme($matches[3]);

        $syntaxOk = preg_match('/(' . PHP_EOL . ')*' . $sectionTags['FAQ'] . '(' . PHP_EOL . ')*(.+)(' . PHP_EOL . ')*' . $sectionTags['changelog'] . '/s', $source, $matches);
        if (!$syntaxOk) {
            return false;
        }
        $sections['FAQ'] = stgh_wc_updater_improve_readme($matches[3]);


        $syntaxOk = preg_match('/(' . PHP_EOL . ')*' . $sectionTags['changelog'] . '(' . PHP_EOL . ')*(.+)/s', $source, $matches);
        if (!$syntaxOk) {
            return false;
        }
        $sections['changelog'] = stgh_wc_updater_improve_readme($matches[3]);

        return $sections;

    }


}

if (!function_exists('stgh_kb_settings_generate')) {
	function stgh_kb_settings_generate() {

		if(isset($_REQUEST['page']) && $_REQUEST['page'] != 'kb') {
			return;
		}

		if ( is_activation() ) {
			return;
		}

		if ( ! \StgHelpdesk\Admin\Stg_Helpdesk_Settings_Page::isTitanActivated()  ) {
			/** @noinspection PhpIncludeInspection */
			require_once( STG_HELPDESK_ROOT . 'vendor/gambitph/titan-framework/titan-framework.php' );
		}

		$titan = TitanFramework::getInstance( 'stgh' );


		$container = $titan->createContainer( array(
				'name'       => __( 'Knowledge Base', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME ),
				'title'      => __( 'Knowledge Base', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME ),
				'id'         => 'kb',
				'parent'     => 'edit.php?post_type=' . STG_HELPDESK_POST_TYPE,
				'capability' => 'settings_tickets',
				'type'       => 'admin-page'
			)
		);

		$version_object = stgh_get_current_version_object();
		$settingsPage = $version_object->getKBSettings();

		foreach ($settingsPage as $tab => $content) {
			/* Add a new tab */

			$tab = $container->createTab(array(
					'name' => $content['name'],
					'title' => isset($content['title']) ? $content['title'] : $content['name'],
					'id' => $tab
				)
			);

			/* Add all options to current tab */
			foreach (array_merge($content['options'], (isset($content['stgh-no-save-button']) ? array() : array(array('type' => 'save')))) as $option) {
				$tab->createOption($option);
			}
		}


	}
}

if (!function_exists('stgh_get_kb_settings')) {
	function stgh_get_kb_settings() {
		$settings = array(
			'kb' => array(
				'name' => __('Options', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
				'options' => array(
					array(
						'name' => __('Column count', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_colcount',
						'type' => 'number',
						'min' => 1,
						'max' => 3,
						'default' => '2'
					),
					array(
						'name' => __('Search width', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_search_width',
						'type' => 'number',
						'min' => 50,
						'max' => 100,
						'step' => 10,
						'unit' => '%',
						'default' => '100'
					),
					array(
						'name' => __('Round corners', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_rounded',
						'type' => 'checkbox',
						'default' => 'false',
					),
					array(
						'name' => __('Articles header color', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_art_header_color',
						'type' => 'color',
						'default' => '#ababab'
					),
					array(
						'name' => __('Articles header font color', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_art_font_header_color',
						'type' => 'color',
						'default' => '#000000'
					),
					array(
						'name' => __('Articles header font size', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_art_font_header_size',
						'type' => 'number',
						'unit' => 'px',
						'max' => 40,
						'default' => 16
					),
					array(
						'name' => __('Articles color', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_art_color',
						'type' => 'color',
						'default' => '#bebebe'
					),
					array(
						'name' => __('Articles font color', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_art_font_color',
						'type' => 'color',
						'default' => '#000000'
					),
					array(
						'name' => __('Articles font size', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_art_font_size',
						'type' => 'number',
						'unit' => 'px',
						'max' => 40,
						'default' => 14
					),
					array(
						'name' => __('Search placeholder', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_search_placeholder',
						'type' => 'text',
						'default' => __('Search', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME)
					)
				)
			),
		);

		return $settings;
	}
}

if (!function_exists('stgh_get_kb_settings_36')) {
	function stgh_get_kb_settings_36() {
		$settings = array(
			'kb' => array(
				'name' => __('Options', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
				'options' => array(
					array(
						'name' => __('Column count', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_colcount',
						'type' => 'number',
						'min' => 1,
						'max' => 3,
						'default' => '2'
					),
					array(
						'name' => __('Search width', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_search_width',
						'type' => 'number',
						'min' => 50,
						'max' => 100,
						'step' => 10,
						'unit' => '%',
						'default' => '100'
					),
					array(
						'name' => __('Round corners', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_rounded',
						'type' => 'checkbox',
						'default' => 'false',
					),
					array(
						'name' => __('Articles header color', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_art_header_color',
						'type' => 'text',
						'default' => '#ababab'
					),
					array(
						'name' => __('Articles header font color', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_art_font_header_color',
						'type' => 'text',
						'default' => '#000000'
					),
					array(
						'name' => __('Articles header font size', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_art_font_header_size',
						'type' => 'number',
						'unit' => 'px',
						'max' => 40,
						'default' => 16
					),
					array(
						'name' => __('Articles color', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_art_color',
						'type' => 'text',
						'default' => '#bebebe'
					),
					array(
						'name' => __('Articles font color', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_art_font_color',
						'type' => 'text',
						'default' => '#000000'
					),
					array(
						'name' => __('Articles font size', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_art_font_size',
						'type' => 'number',
						'unit' => 'px',
						'max' => 40,
						'default' => 14
					),
					array(
						'name' => __('Search placeholder', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME),
						'id' => 'kb_search_placeholder',
						'type' => 'text',
						'default' => __('Search', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME)
					)
				)
			),
		);

		return $settings;
	}
}

if (!function_exists('stgh_get_kb_localize')) {
	function stgh_get_kb_localize() {

		$colCount = stgh_get_option('kb_colcount');
		$rounded = (bool) stgh_get_option('kb_rounded');

		$artHeaderColor = stgh_get_option('kb_art_header_color');
		$artFontHeaderColor = stgh_get_option('kb_art_font_header_color');

		$artColor = stgh_get_option('kb_art_color');
		$artFontColor = stgh_get_option('kb_art_font_color');

		$kb_search_placeholder = stgh_get_option('kb_search_placeholder',__('Search', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME));

		$kb_search_width = stgh_get_option('kb_search_width',100);

		$artFontSize = stgh_get_option('kb_art_font_size');
		$artFontHeaderSize = stgh_get_option('kb_art_font_header_size');

		return array(
			'colCount' => $colCount,
			'rounded' => $rounded,
			'artHeaderColor' => $artHeaderColor,
			'artFontHeaderColor' => $artFontHeaderColor,
			'artColor' => $artColor,
			'artFontColor' => $artFontColor,
			'kb_search_placeholder' => $kb_search_placeholder,
			'kb_search_width' => $kb_search_width,
			'artFontSize' => $artFontSize,
			'artFontHeaderSize' => $artFontHeaderSize
		);

	}
}

if (!function_exists('getKbPage')) {
	function getKbPage() {
		global $stgh_kbPage;

		if ( ! empty( $stgh_kbPage ) ) {
			return $stgh_kbPage;
		}

		$args           = array(
			's' => '['.STG_CATCHERS_HELPDESK_ADDON6_SHORTCODE.']',
			'post_type' => array( 'post', 'page' )
		);
		$query = new WP_Query( $args );
		if ( $query->have_posts() ) {
			while ( $query->have_posts() ) {
				$query->the_post();
				$GLOBALS['stgh_kbPage'] = get_permalink();
			}
		}
		wp_reset_postdata();
		return $GLOBALS['stgh_kbPage'];
	}
}


