<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://mycatchers.com
 * @since      1.0.0
 *
 * @package    Catchers_Helpdesk_Addon6
 * @subpackage Catchers_Helpdesk_Addon6/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Catchers_Helpdesk_Addon6
 * @subpackage Catchers_Helpdesk_Addon6/public
 * @author     mycatchers <support@mycatchers.com>
 */
class Catchers_Helpdesk_Addon6_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/catchers-helpdesk-addon6-public.css', array(), $this->version, 'all' );
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		wp_enqueue_script( 'stgh_kb_public', plugin_dir_url( __FILE__ ) . 'js/catchers-helpdesk-addon6-public.js', array( 'jquery' ), $this->version, false );

		wp_localize_script('stgh_kb_public', 'stghKbLocale', stgh_get_kb_localize());

	}

}
