<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://mycatchers.com
 * @since      1.0.0
 *
 * @package    Catchers_Helpdesk_Addon5
 * @subpackage Catchers_Helpdesk_Addon5/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Catchers_Helpdesk_Addon5
 * @subpackage Catchers_Helpdesk_Addon5/admin
 */
class Catchers_Helpdesk_Addon5_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	public static $nonceAction = 'stgh_catchers_helpdesk_addon5_update_nonce';
	public static $nonceName = 'stgh_catchers_helpdesk_addon5_nonce';

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
        global $current_screen;
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Catchers_Helpdesk_Addon5_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Catchers_Helpdesk_Addon5_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
        if ($current_screen->id == "stgh_ticket_page_reports") {
            wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/catchers-helpdesk-addon5-admin.css', array(), $this->version, 'all');
        }

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		global $current_screen;
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Catchers_Helpdesk_Addon5_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Catchers_Helpdesk_Addon5_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/catchers-helpdesk-addon5-admin.js', array( 'jquery' ), $this->version, false );

        if ($current_screen->id == "stgh_ticket_page_reports") {

            wp_enqueue_script('jquery-ui-datepicker');
            wp_enqueue_style('jqueryui', plugin_dir_url( __FILE__ ) . '/css/jquery-ui.css', false, null );

            wp_enqueue_script( 'stgh_reports', plugin_dir_url( __FILE__ ) . '/js/catchers-helpdesk-addon5-reports.js', array( 'jquery' ), $this->version, false );

            wp_enqueue_script( 'stgh_reports_charts', plugin_dir_url( __FILE__ ) . '/js/Chart.min.js', array( 'stgh_reports' ), $this->version, false );
            wp_enqueue_script( 'stgh_reports_heatmap', plugin_dir_url( __FILE__ ) . '/js/Chart.HeatMap.js', array( 'stgh_reports_charts' ), $this->version, false );


            wp_localize_script('stgh_reports', 'stghReports', array(
                'allCategories' => __('All Categories', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
                'loaderurl' => STG_HELPDESK_URL . "images/ajax-loader.gif",
                'afrt' => _x('Average first response time', 'label for reports indicator', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
                'closedTickets' => _x('Closed tickets', 'label for reports indicator', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
                'totalReplies' => _x('Total replies', 'label for reports indicator', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
                'thegood' => _x('Good', 'label for reports indicator', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
                'thebad' => _x('Bad', 'label for reports indicator', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
                'theneutral' => _x('Neutral', 'label for reports indicator', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
                'undef' => _x('Not set', 'label of undefined indicator', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
                'nodata' => _x('No Data', 'label of undefined indicator', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
                'stgh_answered' => _x('Answered', 'Ticket status', STG_HELPDESK_TEXT_DOMAIN_NAME),
                'stgh_notanswered' => _x('Not answered', 'Ticket status', STG_HELPDESK_TEXT_DOMAIN_NAME),
                'stgh_new' => _x('New', 'Ticket status', STG_HELPDESK_TEXT_DOMAIN_NAME),
                'stgh_closed' => _x('Closed', 'Ticket status', STG_HELPDESK_TEXT_DOMAIN_NAME),
                'stgh_pending' => _x('Pending', 'Ticket status', STG_HELPDESK_TEXT_DOMAIN_NAME),
                'username' => _x('Username', 'Reports table header', STG_HELPDESK_TEXT_DOMAIN_NAME),

            ));
        }

	}

}
