(function( $ ) {

    window.chartsColors = [
        'rgb(255, 99, 132)',
        'rgb(255, 159, 64)',
        'rgb(255, 205, 86)',
        'rgb(75, 192, 192)',
        'rgb(54, 162, 235)',
        'rgb(153, 102, 255)',
        'rgb(103, 102, 255)',
        'rgb(53, 12, 255)',
        'rgb(253, 102, 25)',
        'rgb(15, 10, 25)',
        'rgb(201, 203, 207)'
    ];

    function createAjaxLoaderImg(){
        var loaderImg = document.createElement('img');
        loaderImg.setAttribute('src', stghReports.loaderurl);
        loaderImg.setAttribute('border', '0');
        loaderImg.setAttribute('style', 'width:18px;height:18px;position:relative;left:20px;top:5px;');
        loaderImg.setAttribute('class', 'stgh-reports-button-spinner');

        return loaderImg;
    }

    function loaderSpinnerShowCursor(node){
        node.css( 'cursor', 'progress' );
        node.prop( "disabled", true );
    }

    function loaderSpinnerHideCursor(node){
        node.css( 'cursor', 'default' );
        node.prop( "disabled", false );
    }


    function loaderSpinnerShowBlock(node){
        var loader = createAjaxLoaderImg();
        node.append(loader);
    }

    function loaderSpinnerHideBlock(){
        $( ".stgh-reports-button-spinner" ).remove();
    }


    function loadCategories(){
        var select = $("#stgh_categories_filter");
        loaderSpinnerShowCursor(select);
        $.post(ajaxurl, {action:"stgh_get_categories"},
            function(response) {
                select.empty();
                select.append('<option value="0" selected="selected">'+stghReports.allCategories+'</option>');
                for (var i=0; i<response.length; i++) {
                    select.append('<option value="' + response[i].id + '">' + response[i].desc + '</option>');
                }
                loaderSpinnerHideCursor(select);
            }, "json")
        .fail(function(response) {
            alert('Error: ' + response.responseText);
            loaderSpinnerHideCursor(select);
        });
    }

    function filterHandler(target){
        var span_class = target.attr('class');
        var period = target.attr('data-period');

        if(span_class === 'current')
            return;

        var now = new Date();
        var start = null;

        switch (period){
            case 'year':
                start = new Date(new Date().setFullYear(new Date().getFullYear() - 1));
                break;
            case 'month':
                start = new Date(new Date().setMonth(new Date().getMonth() - 1));
                break;
            case 'week':
                start = new Date(new Date().setDate(new Date().getDate() - 7));
                break;
            case 'day':
                start = new Date(new Date().setDate(new Date().getDate() - 1));
                break;
        }

        $('#stgh_report_date_end').datepicker('setDate', now);
        $('#stgh_report_date_start').datepicker('setDate', start);

        $('.stgh_date_period span').removeClass('current');
        target.addClass('current');
    }

    function hideIndicators(){
    }

    function loadIndicator(sel,val) {
        $('#'+sel).html(val[0]);

        var selGrow = $('#'+sel+"_grow");

        if (val[1] > 0) {
            selGrow.html('+' + val[1] + '%')
                .removeClass()
                .addClass('grow-triangle')
                .show();
        }
        if (val[1] < 0) {
            selGrow.html(val[1] + '%')
                .removeClass()
                .addClass('diminish-triangle')
                .show();
        }

        if (val[1] === 0) {
            selGrow.html(val[1] + '%')
                .removeClass()
                .addClass('triangle')
                .show();
        }

        if (val[1] === '&#8734;') {
            selGrow.html('')
                .removeClass()
                .addClass('triangle')
                .show();
        }

    }



    function loadReports(){
        var select = $('h2.hndle');
        loaderSpinnerShowBlock(select);


        var dateStart = $('#stgh_report_date_start').datepicker('getDate');
        if(dateStart) {
            dateStart.setMinutes(dateStart.getMinutes() - dateStart.getTimezoneOffset());

            var ddDateStart = (dateStart.getDate() < 10 ? '0' : '') + dateStart.getDate();
            var MMDateStart = ((dateStart.getMonth() + 1) < 10 ? '0' : '') + (dateStart.getMonth() + 1);


            var strDateStart =  dateStart.getFullYear() + "-" + MMDateStart  + "-" + ddDateStart;

        }

        var dateEnd = $('#stgh_report_date_end').datepicker('getDate');
        if(dateEnd) {
            dateEnd.setMinutes(dateEnd.getMinutes() - dateEnd.getTimezoneOffset());

            var ddDateEnd = (dateEnd.getDate() < 10 ? '0' : '') + dateEnd.getDate();
            var MMDateEnd = ((dateEnd.getMonth() + 1) < 10 ? '0' : '') + (dateEnd.getMonth() + 1);


            var strDateEnd =  dateEnd.getFullYear() + "-" + MMDateEnd  + "-" + ddDateEnd;
        }

        var data = {
            'action'   : 'stgh_get_reports_data',
            'stgh_dateStart': strDateStart,
            'stgh_dateEnd'  : strDateEnd,
            'stgh_category' : $('#stgh_categories_filter').val(),
            'stgh_clear_cache' : $('#stgh_clear_cache').is(':checked')
        };

        $.post(ajaxurl, data,
            function(response) {

                hideIndicators();

                loadIndicator('new_tickets',response.newTickets);
                loadIndicator('average_first',response.averageFirst);
                loadIndicator('closed_tickets',response.closedTickets);
                loadIndicator('average_closing',response.averageClosing);
                loadIndicator('total_replies',response.totalReplies);
                loadIndicator('new_contact',response.newContacts);

                //loadManagers(response.managers);
                loadManagersTable(response.managers);

                loaderSpinnerHideBlock(select);

                loadCharts(response.charts);
            })
            .fail(function(response) {
                alert('Error: ' + response.responseText);
                loaderSpinnerHideBlock(select);
            });
    }

    /*
    function addManagerIndicator(headerStr, values, parentEl){
        var cell  = document.createElement('div');
        cell.setAttribute('class','stgh-manager-indicator');
        cell.innerHTML = '<div class="stgh-report-result-cell-header">'+headerStr+'</div>';

        var cellcontent = document.createElement('div');
        cellcontent.setAttribute('class','stgh-report-result-manager-cell-content');

        var cellcontentId = document.createElement('div');
        cellcontentId.setAttribute('class','indicator');
        cellcontentId.innerHTML = values[0];

        cellcontent.appendChild(cellcontentId);

        var growClass = 'triangle';
        var valStr = values[1];

        if (values[1] > 0) {
            valStr = '+' + values[1] + '%';
            growClass = 'grow-triangle';
        }

        if (values[1] < 0) {
            valStr = values[1] + '%';
            growClass = 'diminish-triangle';
        }

        if (values[1] === 0) {
            valStr = '~';
            growClass = 'triangle';
        }

        if (values[1] === '&#8734;') {
            valStr = values[1];
            growClass = 'triangle';
        }



        var cellcontentGrow = document.createElement('div');
        cellcontentGrow.setAttribute('class',growClass);
        cellcontentGrow.innerHTML = valStr;

        cellcontent.appendChild(cellcontentGrow);

        cell.appendChild(cellcontent);

        parentEl.appendChild(cell);
    }
    */
    function addManagerIndicatorTable(values, parentEl){
        var cell  = document.createElement('td');
        cell.setAttribute('class','stgh-manager-indicator');

        var cellcontent = document.createElement('div');
        cellcontent.setAttribute('class','stgh-report-result-manager-cell-content');
        cellcontent.innerHTML = "<div class='stgh_man_value'>"+values[0]+"</div>";

        var growClass = 'triangle';
        var valStr = values[1];

        if (values[1] > 0) {
            valStr = '+' + values[1] + '%';
            growClass = 'grow-triangle';
        }

        if (values[1] < 0) {
            valStr = values[1] + '%';
            growClass = 'diminish-triangle';
        }

        if (values[1] === 0) {
            valStr = values[1] + '%';
            growClass = 'triangle';
        }

        if (values[1] === '&#8734;') {
            valStr = '';
            growClass = 'triangle';
        }



        var cellcontentGrow = document.createElement('div');
        cellcontentGrow.setAttribute('class',growClass);
        cellcontentGrow.innerHTML = valStr;

        cellcontent.appendChild(cellcontentGrow);

        cell.appendChild(cellcontent);


        parentEl.appendChild(cell);
    }

    function addManagerVoteIndicatorTable(value, parentEl){
        var cell  = document.createElement('td');
        cell.setAttribute('class','stgh-manager-vote-indicator');

        var cellcontent = document.createElement('div');
        cellcontent.setAttribute('class','stgh-report-result-manager-cell-content');

        var cellcontentId = document.createElement('div');
        cellcontentId.setAttribute('class','indicator indicator-vote');
        cellcontentId.innerHTML = value;

        cellcontent.appendChild(cellcontentId);
        cell.appendChild(cellcontent);

        parentEl.appendChild(cell);
    }


    function addManagerIndicatorHead(){
        var  tHead = document.createElement('thead');

        var tHeadRow = document.createElement('tr');

        tHeadRow.innerHTML = '<th class="stgh-manage-column1"></th>' +
        '<th class="stgh-manage-column2">'+stghReports.username+'</th>'+
        '<th class="stgh-manage-column3">'+stghReports.afrt+'</th>'+
        '<th class="stgh-manage-column4">'+stghReports.closedTickets+'</th>'+
        '<th class="stgh-manage-column5">'+stghReports.totalReplies+'</th>'+
        '<th class="stgh-manage-column6 th-vote"><div class="stgh-vote-ind-header stgh-vote-ind-header-2">'+stghReports.thegood+'</div></th>'+
        '<th class="stgh-manage-column7 th-vote"><div class="stgh-vote-ind-header stgh-vote-ind-header-0">'+stghReports.thebad+'</div></th>'+
        '<th class="stgh-manage-column8 th-vote"><div class="stgh-vote-ind-header stgh-vote-ind-header-1">'+stghReports.theneutral+'</div></th>';

        tHead.append(tHeadRow);
        return tHead;
    }

    function loadManagersTable(managers){
        var manMain = $('.stgh-report-result-managers .main');
        var man = $('.stgh-report-result-managers');

        if(managers.length === 0){
            man.hide();
        }else{
            man.show();
            manMain.empty();
        }

        var  table = document.createElement('table');
        table.setAttribute('class',"stgh-list-table wp-list-table widefat fixed striped posts");

        var tHead = addManagerIndicatorHead();
        table.append(tHead);

        var  tBody = document.createElement('tbody');
        tBody.setAttribute('id',"the-list");

        $.each(managers, function(i, manager) {
            var AvatarImg = document.createElement('img');
            AvatarImg.setAttribute('src', manager.avatar);
            AvatarImg.setAttribute('border', '0');
            AvatarImg.setAttribute('class', 'stgh-avatar');

            var managerRow = document.createElement('tr');
            managerRow.setAttribute('class',"hentry");

            var cell1  = document.createElement('td');

            var avaBlock  = document.createElement('div');
            avaBlock.appendChild(AvatarImg);
            cell1.appendChild(avaBlock);
            managerRow.appendChild(cell1);

            var cell2  = document.createElement('td');
            var userName  = document.createElement('p');

            if(manager.userNameLink) {
                userName.innerHTML = '<a href="' + manager.userNameLink + '">' + manager.userName + '</a>';
            }else{
                userName.innerHTML = '' + manager.userName + '';
            }
            cell2.appendChild(userName);
            managerRow.appendChild(cell2);
            tBody.appendChild(managerRow);

            addManagerIndicatorTable(manager.averageFirst, managerRow);
            addManagerIndicatorTable(manager.closedTickets, managerRow);
            addManagerIndicatorTable(manager.totalReplies, managerRow);

            addManagerVoteIndicatorTable(manager.voted.thegood, managerRow);
            addManagerVoteIndicatorTable(manager.voted.thebad, managerRow);
            addManagerVoteIndicatorTable(manager.voted.theneutral, managerRow);

        });

        table.appendChild(tBody);
        manMain.append(table);
    }

    function checkImportCallback(finishId){
        return function(response) {
            if(response.cnt>0) {
                $('#stgh_import_not_finish').show()
                    .children('span').html(response.cnt);

            }else{
                $('#stgh_import_not_finish').hide();
                clearInterval(finishId);
            }
        }
    }

    function checkImportFine(finishId){
        var callBackFunction = checkImportCallback(finishId);

        $.post(ajaxurl, {action:"stgh_get_tasks"}, callBackFunction, "json")
            .fail(function() {
                $('#stgh_import_not_finish').hide();
                clearInterval(finishId);
            });
    }

    function loadCharts(charts) {
        loadChartDoug("stgh-report-result-ticketcategories", charts['categories']['data'],  charts['categories']['labels']);
        loadChartDoug("stgh-report-result-ticketchannels", charts['channels']['data'],  charts['channels']['labels']);
        loadChartLine("stgh-report-result-opentickets", charts['opentickets']);
        loadChartHeat("stgh-report-result-busiest", charts['heat']['data'],charts['heat']['labels']);
    }

    function loadChartDoug(elId, chartsData, chartsLabels){
        var datasets = [];
        var j=0;
        var legend = '<ul class="stgh-legend">';

        for (var i in chartsData) {
            if (chartsData.hasOwnProperty(i)) {
                var tmp = {};

                var color = '#ececec';
                if(typeof window.chartsColors[j] !== 'undefined')
                {
                    color = window.chartsColors[j];
                }

                tmp.label = chartsLabels[i];
                tmp.color = color;
                tmp.highlight = window.chartsColors[j];
                tmp.value = chartsData[i];

                datasets.push(tmp);
                legend = legend + "<li><span style=\"background-color:" + window.chartsColors[j] + "\"></span>" + chartsLabels[i] + "</li>";
                j++;
            }
        }
        legend = legend+'</ul>';

        var ctx = document.getElementById(elId).getContext("2d");

        window.myDoughnut = window.myDoughnut || {};

        if(window.myDoughnut[elId]) {
            window.myDoughnut[elId].destroy();
            document.getElementById(elId + '-legend').innerHTML = '';
        }

        if (datasets.length > 0) {
            document.getElementById(elId).style.display = "block";
            window.myDoughnut[elId] = new Chart(ctx).Doughnut(datasets, {animateRotate: false ,responsive: false});
            document.getElementById(elId + '-legend').innerHTML = legend;
        } else {

            document.getElementById(elId + '-legend').innerHTML = "";
            document.getElementById(elId).style.display = "none";

            var  nodata = document.createElement('div');
            nodata.setAttribute('class',"stgh_doug_nodata");
            nodata.innerHTML = stghReports.nodata;

            document.getElementById(elId + '-legend').append(nodata);
            // ctx.font = "600 14px Roboto";
            // ctx.fillStyle="#acacac";
            // ctx.fillText(stghReports.nodata,100,100);

        }

    }

    function loadChartLine(elId, chartsData){

        var datasets = [];
        var j=0;
        var legend = '<ul class="stgh-legend">';

        var openColor = {
            'New': '#0073AA',
            'Closed': '#D54E21',
            'Answered': '#26BD43',
            'Pending': '#9f6714',
            'Not answered': '#E8BA2A',
            'Trash': '#000000'};

        for (var i in chartsData) {

            if (chartsData.hasOwnProperty(i)) {
                var tmp = {};
                //tmp.label = i;
                tmp.label = stghReports[i];
                tmp.pointColor = openColor[stghReports[i]];
                tmp.strokeColor = openColor[stghReports[i]];
                tmp.data = chartsData[i]['data'];

                datasets.push(tmp);
                var labels = chartsData[i]['labels'];
                legend = legend + "<li><span style=\"background-color:" + openColor[stghReports[i]] + "\"></span>" + stghReports[i] + "</li>";
                j++;
            }
        }
        legend = legend+'</ul>';

        var lineChartData = {
            labels: labels,
            datasets: datasets
        };


        var ctx = document.getElementById(elId).getContext("2d");

        if(window.myLine) {
            window.myLine.destroy();
            document.getElementById(elId+'-legend').innerHTML = '';
        }


        window.myLine = new Chart(ctx).Line(lineChartData, {
            showScale: true,
            pointDot : true,
            responsive: false,
            bezierCurve : false,
            bezierCurveTension : 0,
            multiTooltipTemplate: "<%= value+' ('+datasetLabel+')' %>",
            datasetFill: false
        });

        document.getElementById(elId+'-legend').innerHTML = legend;

    }

    function loadChartHeat(elId, chartsData, chartsLabels){

        var datasets = [];

        for (var i in chartsData) {
            if (chartsData.hasOwnProperty(i)) {
                var tmp = {};
                tmp.label = i;
                tmp.data = chartsData[i];

                datasets.push(tmp);
            }
        }

        var data = {
            labels : chartsLabels,
            datasets : datasets
        };

        var options = {
            colors: [ "#b5e0f8", "#006799"],
            labelScale: 0.5,
            colorInterpolation: "gradient",
            labelFontColor: "white",
            responsive: false

        };

        if(window.myHeat) {
            window.myHeat.destroy();
        }

        var ctx = document.getElementById(elId).getContext("2d");
        window.myHeat = new Chart(ctx).HeatMap(data, options);

    }

    $(document).ready(function(){

        var finishId = setInterval(function(){
            checkImportFine(finishId);
        }, 5000);

        loadCategories();

        $('#stgh_custom_date').live('click',function(){
            $('#stgh_custom_date_row').toggleClass('stgh_custom_date_row_show');
        });

        $('#stgh_report_date_start').ready(function () {
            var oneWeekAgo = new Date();
            oneWeekAgo.setDate(oneWeekAgo.getDate() - 7);

            var now = new Date();

            $('#stgh_report_date_start').datepicker('setDate', oneWeekAgo);
            $('#stgh_report_date_end').datepicker('setDate', now);

            loadReports();
        });


        $('.stgh_date_period span[id != "stgh_custom_date"]').live('click',function(event){
            filterHandler($(event.target));
            loadReports();
        });

        $('#show-reports').live('click', function(){
            loadReports();
        });

        $('.handlediv').live('click',function(){
            $(this).parents('.postbox').toggleClass('closed');
        });

    });

})( jQuery );