<?php

/**
 * @param $dateStart
 * @param $dateEnd
 * @param $time
 * @return bool
 */
function stgh_check_interval($dateStart,$dateEnd,$time){

    if($dateEnd && $time > $dateEnd || $dateEnd === false)
        return false;

    if($dateStart && $time < $dateStart)
        return false;

    return true;
}

/**
 * @param $ticketStats
 * @param $category
 * @return bool
 */
function stgh_check_category($ticketStats, $category)
{
    if (!isset($ticketStats['ticket_category'])) {
        return false;
    }

    if ($ticketStats['ticket_category'] != $category) {
        return false;
    }

    return true;
}

/**
 * @param $mysqli mysqli
 * @param $dateStart
 * @param $dateEnd
 * @return int
 */

function stgh_reports_ufilter_reducer($mysqli, $dateStart, $dateEnd){
    global $wpdb;

    $uEventTable = $wpdb->prefix.STG_CATCHERS_HELPDESK_ADDON5_UTABLENAME;

    if(!empty($dateStart) && !empty($dateStart)) {
        $whereDateInterval = "UNIX_TIMESTAMP(created_at) BETWEEN {$dateStart} AND {$dateEnd}";
    }else{
        if(empty($dateStart)){
            $whereDateInterval = "UNIX_TIMESTAMP(created_at) <= {$dateEnd}";
        }else{
            $whereDateInterval = "UNIX_TIMESTAMP(created_at) >= {$dateStart}";
        }
    }
    $acc=0;

    $query = "SELECT * from {$uEventTable} WHERE {$whereDateInterval} ORDER BY created_at DESC";
    if ($result = $mysqli->query($query,MYSQLI_USE_RESULT)) {
        while ($row = $result->fetch_assoc()) {
            $acc++;
        }
        $result->free();
    }
    return $acc;

}


/**
 * @param $mysqli mysqli
 * @param $dateStart
 * @param $dateEnd
 * @param $category
 * @return array
 */

function stgh_reports_filter_reducer($mysqli, $dateStart, $dateEnd, $category,$stghClearCache = false){

    $ckey = stgh_report_get_interval($dateStart,$dateEnd)."-".$category."-";

    if ( $stghClearCache == "true" )
        delete_transient($ckey);

    $cached = get_transient($ckey);

    if ( $cached !== false)
        return $cached;


    $newTickets = stgh_reports_newTickets_reducer($mysqli, $dateStart, $dateEnd, $category);

    $totalReplies = stgh_reports_totalRepliesByAgent_reducer($mysqli, $dateStart, $dateEnd, $category);

    $totalReplies['acc'] = stgh_reports_totalReplies_reducer($mysqli, $dateStart, $dateEnd, $category);

    $categories = stgh_reports_categories_reducer($mysqli, $dateStart, $dateEnd, $category);

    $channels = stgh_reports_channels_reducer($mysqli, $dateStart, $dateEnd, $category);

    $heat = stgh_reports_heat_reducer($mysqli, $dateStart, $dateEnd, $category);

    $afr = stgh_reports_afr_reducer($mysqli, $dateStart, $dateEnd, $category);

    $act = stgh_reports_act_reducer($mysqli, $dateStart, $dateEnd, $category);

    $closedTickets['acc'] = stgh_reports_closed_reducer($mysqli, $dateStart, $dateEnd, $category);

    $openTickets = stgh_reports_open_reducer($mysqli, $dateStart, $dateEnd, $category);

    $voted  = stgh_reports_voted_reducer($mysqli, $dateStart, $dateEnd, $category);

    $return = array(
        'newTickets' => $newTickets,
        'totalReplies' => $totalReplies,
        'categories' => $categories,
        'channels' => $channels,
        'heat' => $heat,
        'afr' => $afr,
        'act' => $act,
        'closedTickets' => $closedTickets,
        'opentickets' => $openTickets,
        'voted' => $voted);

    set_transient( $ckey, $return, 1 * HOUR_IN_SECONDS );

    return $return;

}



function stgh_report_get_interval($dateStart,$dateEnd)
{
    if (!empty($dateStart) && !empty($dateEnd)) {
        $whereDateInterval = "UNIX_TIMESTAMP(created_at) BETWEEN {$dateStart} AND {$dateEnd}";
    } else {
        if (empty($dateStart)) {
            $whereDateInterval = "UNIX_TIMESTAMP(created_at) <= {$dateEnd}";
        } else {
            $whereDateInterval = "UNIX_TIMESTAMP(created_at) >= {$dateStart}";
        }
    }

    return $whereDateInterval;
}

function stgh_report_get_create_interval($dateStart,$dateEnd)
{
    if (!empty($dateStart) && !empty($dateEnd)) {
        $whereDateInterval = "AND UNIX_TIMESTAMP(ticket_created_at) BETWEEN {$dateStart} AND {$dateEnd}";
    } else {
        if (empty($dateStart)) {
            $whereDateInterval = "AND UNIX_TIMESTAMP(ticket_created_at) <= {$dateEnd}";
        } else {
            $whereDateInterval = "AND UNIX_TIMESTAMP(ticket_created_at) >= {$dateStart}";
        }
    }

    return $whereDateInterval;
}


/**
 * @param $mysqli mysqli
 * @param $dateStart
 * @param $dateEnd
 * @param $category
 * @return array
 */
function stgh_reports_newTickets_reducer($mysqli, $dateStart, $dateEnd, $category){
    global $wpdb;

    $eventTable = $wpdb->prefix.STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;

    $whereDateInterval = stgh_report_get_interval($dateStart,$dateEnd);

    $newTickets = 0;

    $query = "SELECT * from {$eventTable} WHERE {$whereDateInterval} ORDER BY ticket_id, created_at ASC";

    if ($result = $mysqli->query($query, MYSQLI_USE_RESULT)) {

        while ($row = $result->fetch_assoc()) {


            if(!isset($ticketId)) {
                $ticketStats = array();
            }else {

                if ($row['ticket_id'] != $ticketId && isset($ticketStats['new_ticket'])) {

                    $ticketStats['ticket_category'] = (empty($ticketStats['ticket_category'])) ? 'undef' : $ticketStats['ticket_category'];

                    if (isset($ticketStats['ticket_category'])) {
                        if (!($category > 0 && $ticketStats['ticket_category'] != $category)) {
                            $newTickets = $newTickets + 1;
                        }
                    }

                    $ticketStats = array();
                }
            }


            $ticketId = $row['ticket_id'];
            $ticketStats['ticket_category'] = $row['ticket_category'];

            if($row['event_type'] == 'new_ticket') {
                $ticketStats['new_ticket'] = true;
            }
        }

        $ticketStats['ticket_category'] = (empty($ticketStats['ticket_category'])) ? 'undef' : $ticketStats['ticket_category'];

        if (isset($ticketStats['ticket_category'])) {
            if (!($category > 0 && $ticketStats['ticket_category'] != $category)) {
                if(isset($ticketId) && $ticketId > 0)
                    if(isset($ticketStats['new_ticket']))
                        $newTickets = $newTickets + 1;
            }
        }

        $result->free();
    }

    return array('acc' => $newTickets);

}

/**
 * @param $mysqli mysqli
 * @param $dateStart
 * @param $dateEnd
 * @param $category
 * @return int
 */
function stgh_reports_totalReplies_reducer($mysqli, $dateStart, $dateEnd, $category){
    global $wpdb;

    $eventTable = $wpdb->prefix.STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;
    $whereDateInterval = stgh_report_get_interval($dateStart, $dateEnd);

    $totalReplies = 0;

    /** @noinspection SqlResolve **/
    $query = "SELECT * from {$eventTable} WHERE {$whereDateInterval} ORDER BY ticket_id, created_at ASC";

    if ($result = $mysqli->query($query,MYSQLI_USE_RESULT)) {

        while ($row = $result->fetch_assoc()) {

            if(!isset($ticketId) || $row['ticket_id'] != $ticketId) {
                $ticketStats = array();
            }

            $ticketId = $row['ticket_id'];
            $ticketStats['ticket_category'] = $row['ticket_category'];

            switch ($category>0) {
                case true:
                    switch ($row['event_type']) {
                        case 'client_reply':
                        case 'agent_reply':
                                if($category == $ticketStats['ticket_category']) {
                                    $totalReplies = $totalReplies + 1;
                                }
                            continue;
                        default:
                            continue;
                    }
                    break;
                default:
                    switch ($row['event_type']) {
                        case 'client_reply':
                        case 'agent_reply':
                            $totalReplies = $totalReplies+1;
                            continue;
                        default:
                            continue;
                    }
            }
        }
        $result->free();
    }

    return $totalReplies;

}

/**
 * @param $mysqli mysqli
 * @param $dateStart
 * @param $dateEnd
 * @param $category
 * @return array
 */

function stgh_reports_totalRepliesByAgent_reducer($mysqli, $dateStart, $dateEnd, $category){
    global $wpdb;

    $eventTable = $wpdb->prefix.STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;
    $whereDateInterval = stgh_report_get_interval($dateStart, $dateEnd);

    $totalReplies = array();

    /** @noinspection SqlResolve **/
    $query = "SELECT * from {$eventTable} WHERE {$whereDateInterval} ORDER BY ticket_id, created_at ASC";

    if ($result = $mysqli->query($query,MYSQLI_USE_RESULT)) {

        while ($row = $result->fetch_assoc()) {

            if(!isset($ticketId) || $row['ticket_id'] != $ticketId) {
                $ticketStats = array();
            }

            $ticketId = $row['ticket_id'];
            $ticketStats['ticket_category'] = $row['ticket_category'];

            switch ($category>0) {
                case true:
                    switch ($row['event_type']) {
                        case 'agent_reply':
                            if($category == $ticketStats['ticket_category']) {

                                if(!empty($row['ticket_assignee'])) {
                                    if (!isset($totalReplies[$row['ticket_assignee']]['count'])) {
                                        $totalReplies[$row['ticket_assignee']]['count'] = 0;
                                    }
                                    $totalReplies[$row['ticket_assignee']]['count'] = $totalReplies[$row['ticket_assignee']]['count'] + 1;
                                }
                            }
                            continue;
                        default:
                            continue;
                    }
                    break;
                default:
                    switch ($row['event_type']) {
                        case 'agent_reply':
                            if(!empty($row['ticket_assignee'])) {
                                if (!isset($totalReplies[$row['ticket_assignee']]['count'])) {
                                    $totalReplies[$row['ticket_assignee']]['count'] = 0;
                                }
                                $totalReplies[$row['ticket_assignee']]['count'] = $totalReplies[$row['ticket_assignee']]['count'] + 1;
                            }
                            continue;
                        default:
                            continue;
                    }
            }
        }
        $result->free();
    }

    return $totalReplies;

}

/**
 * @param $mysqli mysqli
 * @param $dateStart
 * @param $dateEnd
 * @param $category
 * @return array
 */

function stgh_reports_categories_reducer($mysqli, $dateStart, $dateEnd, $category){
    global $wpdb;

    $eventTable = $wpdb->prefix.STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;
    $whereDateInterval = stgh_report_get_interval($dateStart, $dateEnd);
    $whereCreateDateInterval = stgh_report_get_create_interval($dateStart, $dateEnd);

    $categoriesTmp = stgh_ticket_get_categories();
    $categoryNames = array('undef' => _x('Not set', 'label of undefined indicator', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME));
    foreach ($categoriesTmp as $categoryTmp)
    {
        $categoryNames[$categoryTmp->term_id] = $categoryTmp->name;
    }

    $categories = array();

    /** @noinspection SqlResolve **/
    $query = "SELECT * from {$eventTable} WHERE {$whereDateInterval} {$whereCreateDateInterval} ORDER BY ticket_id, created_at ASC";

    if ($result = $mysqli->query($query,MYSQLI_USE_RESULT)) {

        while ($row = $result->fetch_assoc()) {
            if(!isset($ticketId)) {
                $ticketStats = array();
            }else {

                if ($row['ticket_id'] != $ticketId) {

                    $ticketStats['ticket_category'] = (empty($ticketStats['ticket_category'])) ? 'undef' : $ticketStats['ticket_category'];

                    if (isset($ticketStats['ticket_category'])) {
                        if (!($category > 0 && $ticketStats['ticket_category'] != $category)) {
                            $categories[$ticketStats['ticket_category']]['count'] = $categories[$ticketStats['ticket_category']]['count'] + 1;

                            if (!isset($categories[$ticketStats['ticket_category']]['name']))
                                $categories[$ticketStats['ticket_category']]['name'] = $categoryNames[$ticketStats['ticket_category']];
                        }
                    }

                    $ticketStats = array();
                }
            }

            $ticketId = $row['ticket_id'];

            $ticketStats['ticket_category'] = $row['ticket_category'];

        }


        $ticketStats['ticket_category'] = (empty($ticketStats['ticket_category'])) ? 'undef' : $ticketStats['ticket_category'];

        if(isset($ticketId) && $ticketId > 0) {
            if (isset($ticketStats['ticket_category'])) {
                if (!($category > 0 && $ticketStats['ticket_category'] != $category)) {
                    $categories[$ticketStats['ticket_category']]['count'] = $categories[$ticketStats['ticket_category']]['count'] + 1;

                    if (!isset($categories[$ticketStats['ticket_category']]['name']))
                        $categories[$ticketStats['ticket_category']]['name'] = $categoryNames[$ticketStats['ticket_category']];
                }
            }
        }

        $result->free();
    }

    return $categories;

}

/**
 * @param $mysqli mysqli
 * @param $dateStart
 * @param $dateEnd
 * @param $category
 * @return array
 */

function stgh_reports_channels_reducer($mysqli, $dateStart, $dateEnd, $category){
    global $wpdb;

    $eventTable = $wpdb->prefix.STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;
    $whereDateInterval = stgh_report_get_interval($dateStart, $dateEnd);
    $whereCreateDateInterval = stgh_report_get_create_interval($dateStart, $dateEnd);

    $channels = array();

    /** @noinspection SqlResolve **/
    $query = "SELECT * from {$eventTable} WHERE {$whereDateInterval} {$whereCreateDateInterval} ORDER BY ticket_id, created_at ASC";

    if ($result = $mysqli->query($query,MYSQLI_USE_RESULT)) {

        while ($row = $result->fetch_assoc()) {

            if(!isset($ticketId)) {
                $ticketStats = array();
            }else {

                if ($row['ticket_id'] != $ticketId) {
                    $ticketStats['ticket_category'] = (empty($ticketStats['ticket_category'])) ? 'undef' : $ticketStats['ticket_category'];

                    if (isset($ticketStats['ticket_category'])) {
                        if (!($category > 0 && $ticketStats['ticket_category'] != $category)) {
                            if (empty($ticketStats['ticket_channel'])) {
                                $ticketStats['ticket_channel'] = _x('Not set', 'label of undefined indicator', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME);
                            }

                            if (is_email($ticketStats['ticket_channel'])) {
                                $ticketStats['ticket_channel'] = _x('email', 'label of undefined indicator', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME);
                            }

                            if (!isset($channels[$ticketStats['ticket_channel']]))
                                $channels[$ticketStats['ticket_channel']]['count'] = 0;

                            $channels[$ticketStats['ticket_channel']]['count'] = $channels[$ticketStats['ticket_channel']]['count'] + 1;
                            $channels[$ticketStats['ticket_channel']]['name'] = $ticketStats['ticket_channel'];
                        }
                    }

                    $ticketStats = array();
                }
            }

            $ticketId = $row['ticket_id'];
            $ticketStats['ticket_category'] = $row['ticket_category'];

            if($row['event_type'] == 'new_ticket')
                $ticketStats['ticket_channel'] = $row['ticket_channel'];


        }

        if(isset($ticketId) && $ticketId > 0) {
            $ticketStats['ticket_category'] = (empty($ticketStats['ticket_category'])) ? 'undef' : $ticketStats['ticket_category'];

            if (isset($ticketStats['ticket_category'])) {
                if (!($category > 0 && $ticketStats['ticket_category'] != $category)) {
                    if (empty($ticketStats['ticket_channel'])) {
                        $ticketStats['ticket_channel'] = _x('Not set', 'label of undefined indicator', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME);
                    }

                    if (is_email($ticketStats['ticket_channel'])) {
                        $ticketStats['ticket_channel'] = _x('email', 'label of undefined indicator', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME);
                    }

                    if (!isset($channels[$ticketStats['ticket_channel']]))
                        $channels[$ticketStats['ticket_channel']]['count'] = 0;


                    $channels[$ticketStats['ticket_channel']]['count'] = $channels[$ticketStats['ticket_channel']]['count'] + 1;
                    $channels[$ticketStats['ticket_channel']]['name'] = $ticketStats['ticket_channel'];

                }
            }
        }
        $result->free();
    }

    return $channels;

}

/**
 * @param $mysqli mysqli
 * @param $dateStart
 * @param $dateEnd
 * @param $category
 * @return array
 */
function stgh_reports_heat_reducer($mysqli, $dateStart, $dateEnd, $category){
    global $wpdb;

    $eventTable = $wpdb->prefix.STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;
    $whereDateInterval = stgh_report_get_interval($dateStart, $dateEnd);
    $whereCreateDateInterval = stgh_report_get_create_interval($dateStart, $dateEnd);


    $heatHours = array(
        _x('12-2a', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('2-4a', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('4-6a', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('6-8a', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('8-10a', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('10-12a', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('12-2p', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('2-4p', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('4-6p', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('6-8p', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('8-10p', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('10-12p', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME)
    );

    $heatDays = array(
        _x('Mo', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('Tu', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('We', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('Th', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('Fr', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('Sa', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
        _x('Su', 'time label for reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
    );

    $periodInHours = 2;

    $heat = array();

    foreach($heatDays as $day)
    {
        $heat[$day] = array(0,0,0,0,0,0,0,0,0,0,0,0);
    }

    /** @noinspection SqlResolve */
    $query = "SELECT *, HOUR(created_at) as hr, WEEKDAY(created_at) as dy from {$eventTable} WHERE {$whereDateInterval} {$whereCreateDateInterval} ORDER BY ticket_id, created_at ASC";

    if ($result = $mysqli->query($query,MYSQLI_USE_RESULT)) {

        while ($row = $result->fetch_assoc()) {

            if(!isset($ticketId)) {
                $ticketStats = array();
            }else {

                if ($row['ticket_id'] != $ticketId) {

                    $ticketStats['ticket_category'] = (empty($ticketStats['ticket_category'])) ? 'undef' : $ticketStats['ticket_category'];

                    if (isset($ticketStats['ticket_category'])) {
                        if (!($category > 0 && $ticketStats['ticket_category'] != $category)) {
                            $heatHourIndex = (int)floor($ticketStats['new_ticket_hr'] / $periodInHours);
                            $heatDayIndex = $ticketStats['new_ticket_dy'];
                            $heat[$heatDays[$heatDayIndex]][$heatHourIndex] = $heat[$heatDays[$heatDayIndex]][$heatHourIndex] + 1;
                        }
                    }

                    $ticketStats = array();
                }
            }


            $ticketId = $row['ticket_id'];
            $ticketStats['ticket_category'] = $row['ticket_category'];

            if($row['event_type'] == 'new_ticket') {
                $ticketStats['new_ticket_hr'] = $row['hr'];
                $ticketStats['new_ticket_dy'] = $row['dy'];
            }
        }


        $ticketStats['ticket_category'] = (empty($ticketStats['ticket_category'])) ? 'undef' : $ticketStats['ticket_category'];

        if (isset($ticketStats['ticket_category'])) {
            if (!($category > 0 && $ticketStats['ticket_category'] != $category)) {
                if(isset($ticketId) && $ticketId > 0)
                {
                    $heatHourIndex = (int)floor($ticketStats['new_ticket_hr'] / $periodInHours);
                    $heatDayIndex = $ticketStats['new_ticket_dy'];

                    $heat[$heatDays[$heatDayIndex]][$heatHourIndex] = $heat[$heatDays[$heatDayIndex]][$heatHourIndex] + 1;
                }

            }
        }

        $result->free();

        /*
        while ($row = $result->fetch_assoc()) {

            if(!isset($ticketId) || $row['ticket_id'] != $ticketId) {
                $ticketStats = array();
            }

            $ticketId = $row['ticket_id'];
            $ticketStats['ticket_category'] = $row['ticket_category'];

            switch ($category>0) {
                case true:
                    switch ($row['event_type']) {
                        case 'agent_reply':
                        case 'client_reply':
                                if ($ticketStats['ticket_category'] == $category) {

                                    $heatHourIndex = (int)floor($row['hr'] / $periodInHours);
                                    $heatDayIndex = $row['dy'];

                                    $heat[$heatDays[$heatDayIndex]][$heatHourIndex] = $heat[$heatDays[$heatDayIndex]][$heatHourIndex] + 1;
                                }
                        continue;
                        default:
                            continue;
                    }
                    break;
                default:
                    switch ($row['event_type']) {
                        case 'agent_reply':
                        case 'client_reply':
                                $heatHourIndex = (int)floor($row['hr'] / $periodInHours);
                                $heatDayIndex = $row['dy'];

                                $heat[$heatDays[$heatDayIndex]][$heatHourIndex] = $heat[$heatDays[$heatDayIndex]][$heatHourIndex] + 1;
                            continue;
                        default:
                            continue;
                    }
            }
        }
        $result->free();
        */
    }
    //d($heat);
    $heat['heatHours']['labels'] = $heatHours;
    return $heat;

}

/**
 * @param $mysqli mysqli
 * @param $dateStart
 * @param $dateEnd
 * @param $category
 * @return array
 */
function stgh_reports_afr_reducer($mysqli, $dateStart, $dateEnd, $category){
    global $wpdb;

    $eventTable = $wpdb->prefix.STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;
    $whereDateInterval = stgh_report_get_interval($dateStart, $dateEnd);

    $afr = array();


    /** @noinspection SqlResolve **/
    $query = "SELECT * from {$eventTable} WHERE {$whereDateInterval} ORDER BY ticket_id, created_at ASC";

    if ($result = $mysqli->query($query,MYSQLI_USE_RESULT)) {

        while ($row = $result->fetch_assoc()) {

            if(!isset($ticketId) || $row['ticket_id'] != $ticketId) {
                $ticketStats = array();
            }

            $ticketId = $row['ticket_id'];
            $ticketStats['ticket_category'] = $row['ticket_category'];

            switch ($category>0) {
                case true:
                    switch ($row['event_type']) {
                        case 'agent_reply':
                            if ($ticketStats['ticket_category'] == $category) {

                                if (!isset($ticketStats['first_agent_reply'][$row['ticket_assignee']])) {
                                    $ticketStats['first_agent_reply'][$row['ticket_assignee']] = $row['created_at'];

                                    $ticketAverage = strtotime($ticketStats['first_agent_reply'][$row['ticket_assignee']]) - strtotime($row['ticket_created_at']);

                                    $afr[$row['ticket_assignee']]['acc'] = (isset($afr[$row['ticket_assignee']]['acc'])) ? $afr[$row['ticket_assignee']]['acc'] + $ticketAverage : $ticketAverage;
                                    $afr[$row['ticket_assignee']]['count'] = (isset($afr[$row['ticket_assignee']]['count'])) ? $afr[$row['ticket_assignee']]['count'] + 1 : 1;
                                    $afr['countAll'] = (isset($afr['countAll'])) ? $afr['countAll'] + 1 : 1;

                                }
                            }
                            continue;
                        default:
                            continue;
                    }
                    break;
                default:
                    switch ($row['event_type']) {
                        case 'agent_reply':

                            if (!isset($ticketStats['first_agent_reply'][$row['ticket_assignee']])) {

                                $ticketStats['first_agent_reply'][$row['ticket_assignee']] = $row['created_at'];

                                $ticketAverage = strtotime($ticketStats['first_agent_reply'][$row['ticket_assignee']]) - strtotime($row['ticket_created_at']);

                                $afr[$row['ticket_assignee']]['acc'] = (isset($afr[$row['ticket_assignee']]['acc'])) ? $afr[$row['ticket_assignee']]['acc'] + $ticketAverage : $ticketAverage;
                                $afr[$row['ticket_assignee']]['count'] = (isset($afr[$row['ticket_assignee']]['count'])) ? $afr[$row['ticket_assignee']]['count'] + 1 : 1;
                                $afr['countAll'] = (isset($afr['countAll'])) ? $afr['countAll'] + 1 : 1;

                            }
                            continue;
                        default:
                            continue;
                    }
            }
        }
        $result->free();
    }

    return $afr;

}

/**
 * @param $mysqli mysqli
 * @param $dateStart
 * @param $dateEnd
 * @param $category
 * @return array
 */
function stgh_reports_act_reducer($mysqli, $dateStart, $dateEnd, $category){
    global $wpdb;

    $eventTable = $wpdb->prefix.STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;
    $whereDateInterval = stgh_report_get_interval($dateStart, $dateEnd);

    $act = array();

    /** @noinspection SqlResolve **/
    $query = "SELECT * from {$eventTable} WHERE {$whereDateInterval} ORDER BY ticket_id, created_at ASC";

    if ($result = $mysqli->query($query,MYSQLI_USE_RESULT)) {

        while ($row = $result->fetch_assoc()) {

            if(!isset($ticketId) || $row['ticket_id'] != $ticketId) {
                $ticketStats = array();
            }

            $ticketId = $row['ticket_id'];
            $ticketStats['ticket_category'] = $row['ticket_category'];

            switch ($category>0) {
                case true:
                    switch ($row['event_type']) {
                        case 'change_status':
                                if($ticketStats['ticket_category'] == $category) {

                                    if ($row['ticket_status'] == 'stgh_closed') {

                                        if (!isset($ticketStats['ticket_close'])) {
                                            $ticketStats['ticket_close'] = true;

                                            if(isset($ticketStats['ticket_reopen']))
                                            {
                                                $closedAverage = strtotime($row['created_at']) - $ticketStats['ticket_reopen'];
                                            }else {
                                                $closedAverage = strtotime($row['created_at']) - strtotime($row['ticket_created_at']);
                                            }

                                            $act[$row['ticket_assignee']]['acc'] = (isset($act[$row['ticket_assignee']]['acc'])) ? $act[$row['ticket_assignee']]['acc'] + $closedAverage : $closedAverage;
                                            $act[$row['ticket_assignee']]['count'] = (isset($act[$row['ticket_assignee']]['count'])) ? $act[$row['ticket_assignee']]['count'] + 1 : 1;

                                            $act['countAll'] = (isset($act['countAll'])) ? $act['countAll'] + 1 : 1;
                                        }

                                    }else{
                                        if (isset($ticketStats['ticket_close'])) {
                                            unset($ticketStats['ticket_close']);
                                            $ticketStats['ticket_reopen'] = strtotime($row['created_at']);
                                        }
                                    }
                                }
                            continue;
                        default:
                            continue;
                    }
                    break;
                default:
                    switch ($row['event_type']) {
                        case 'change_status':


                                if ($row['ticket_status'] == 'stgh_closed') {

                                    if (!isset($ticketStats['ticket_close'])) {
                                        $ticketStats['ticket_close'] = true;

                                            if(isset($ticketStats['ticket_reopen']))
                                            {
                                                $closedAverage = strtotime($row['created_at']) - $ticketStats['ticket_reopen'];
                                            }else {
                                                $closedAverage = strtotime($row['created_at']) - strtotime($row['ticket_created_at']);
                                            }

                                        $act[$row['ticket_assignee']]['acc'] = (isset($act[$row['ticket_assignee']]['acc'])) ? $act[$row['ticket_assignee']]['acc'] + $closedAverage : $closedAverage;
                                        $act[$row['ticket_assignee']]['count'] = (isset($act[$row['ticket_assignee']]['count'])) ? $act[$row['ticket_assignee']]['count'] + 1 : 1;

                                        $act['countAll'] = (isset($act['countAll'])) ? $act['countAll'] + 1 : 1;
                                    }

                                }else{
                                    if (isset($ticketStats['ticket_close'])) {
                                        unset($ticketStats['ticket_close']);
                                        $ticketStats['ticket_reopen'] = strtotime($row['created_at']);
                                    }
                                }

                            continue;
                        default:
                            continue;
                    }
            }
        }
        $result->free();
    }

    return $act;

}

/**
 * @param $mysqli mysqli
 * @param $dateStart
 * @param $dateEnd
 * @param $category
 * @return int
 */
function stgh_reports_closed_reducer($mysqli, $dateStart, $dateEnd, $category){
    global $wpdb;

    $eventTable = $wpdb->prefix.STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;
    $whereDateInterval = stgh_report_get_interval($dateStart, $dateEnd);

    $closed = 0;

    /** @noinspection SqlResolve **/
    $query = "SELECT * from {$eventTable} WHERE {$whereDateInterval} ORDER BY ticket_id, created_at ASC";

    if ($result = $mysqli->query($query,MYSQLI_USE_RESULT)) {

        while ($row = $result->fetch_assoc()) {

            if(!isset($ticketId) || $row['ticket_id'] != $ticketId) {
                $ticketStats = array();
            }

            $ticketId = $row['ticket_id'];
            $ticketStats['ticket_category'] = $row['ticket_category'];

            switch ($category>0) {
                case true:
                    switch ($row['event_type']) {
                        case 'change_status':
                            if($ticketStats['ticket_category'] == $category) {

                                if ($row['ticket_status'] == 'stgh_closed') {

                                    if (!isset($ticketStats['ticket_close'])) {
                                        $ticketStats['ticket_close'] = true;
                                        $closed++;
                                    }
                                }
                            }
                            continue;
                        default:
                            continue;
                    }
                    break;
                default:
                    switch ($row['event_type']) {
                        case 'change_status':

                            if ($row['ticket_status'] == 'stgh_closed') {

                                if (!isset($ticketStats['ticket_close'])) {
                                    $ticketStats['ticket_close'] = true;
                                    $closed++;
                                }
                            }

                            continue;
                        default:
                            continue;
                    }
            }
        }
        $result->free();
    }

    return $closed;

}

/**
 * @param $mysqli mysqli
 * @param $dateStart
 * @param $dateEnd
 * @param $category
 * @return array
 */
function stgh_reports_open_reducer($mysqli, $dateStart, $dateEnd, $category){
    global $wpdb;

    $eventTable = $wpdb->prefix.STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;
    $whereDateInterval = stgh_report_get_interval($dateStart, $dateEnd);

    $lineChartsStatuses = array('stgh_new', 'stgh_answered','stgh_notanswered');

    $period = $dateEnd - $dateStart;
    if($period / 86400 < 2)
    {
        $lineChartsPeriodCount = 1;
    }else {
        $lineChartsPeriodCount = 4;

    }
    $lineChartsInterval = ($dateEnd - $dateStart)/$lineChartsPeriodCount;

    $openTickets = array();
    foreach($lineChartsStatuses as $lineChartsStatus)
    {
        $openTickets[$lineChartsStatus] = array();
    }


    /** @noinspection SqlResolve **/
    $query = "SELECT * from {$eventTable} WHERE {$whereDateInterval} ORDER BY ticket_id, created_at ASC";

    if ($result = $mysqli->query($query,MYSQLI_USE_RESULT)) {

        while ($row = $result->fetch_assoc()) {

            if(!isset($ticketId)) {
                $ticketStats = array();
            }else {

                if ($row['ticket_id'] != $ticketId) {

                    $ticketStats['ticket_category'] = (empty($ticketStats['ticket_category'])) ? 'undef' : $ticketStats['ticket_category'];

                    if (isset($ticketStats['ticket_category'])) {
                        if (!($category > 0 && $ticketStats['ticket_category'] != $category)) {
                            if (in_array($ticketStats['ticket_status'], $lineChartsStatuses)) {
                                $intervalNum = floor(($ticketStats['time'] - $dateStart) / $lineChartsInterval);
                                if (!isset($openTickets[$ticketStats['ticket_status']][$intervalNum])) {
                                    $openTickets[$ticketStats['ticket_status']][$intervalNum] = 0;
                                }
                                $openTickets[$ticketStats['ticket_status']][$intervalNum] = $openTickets[$ticketStats['ticket_status']][$intervalNum] + 1;
                            }
                        }
                    }


                    $ticketStats = array();
                }
            }

            $ticketId = $row['ticket_id'];
            $ticketStats['ticket_category'] = $row['ticket_category'];
            $ticketStats['ticket_status'] = $row['ticket_status'];
            $ticketStats['time'] = strtotime($row['created_at']);

        }

        $ticketStats['ticket_category'] = (empty($ticketStats['ticket_category'])) ? 'undef' : $ticketStats['ticket_category'];

        if(isset($ticketId) && $ticketId > 0) {
            if (isset($ticketStats['ticket_category'])) {
                if (!($category > 0 && $ticketStats['ticket_category'] != $category)) {
                    if (in_array($ticketStats['ticket_status'], $lineChartsStatuses)) {
                        $intervalNum = floor(($ticketStats['time'] - $dateStart) / $lineChartsInterval);
                        if (!isset($openTickets[$ticketStats['ticket_status']][$intervalNum])) {
                            $openTickets[$ticketStats['ticket_status']][$intervalNum] = 0;
                        }
                        $openTickets[$ticketStats['ticket_status']][$intervalNum] = $openTickets[$ticketStats['ticket_status']][$intervalNum] + 1;
                    }
                }
            }
        }

        $result->free();
    }

    foreach($openTickets as $status => $values)
    {
        for($i=0;$i<=$lineChartsPeriodCount;$i++)
        {
            $timestamp = $dateStart+$lineChartsInterval*$i;
            if(isset($values[$i]))
            {
                unset($openTickets[$status][$i]);
                $openTickets[$status][$timestamp] = $values[$i];

            }else{
                $openTickets[$status][$timestamp] = 0;

            }

            $openTickets['labels'][date('d/m/Y',$timestamp)] = true;
        }
    }


    return $openTickets;

}

/**
 * @param $mysqli mysqli
 * @param $dateStart
 * @param $dateEnd
 * @param $category
 * @return array
 */
function stgh_reports_voted_reducer($mysqli, $dateStart, $dateEnd, $category){
    global $wpdb;

    $eventTable = $wpdb->prefix.STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;
    $whereDateInterval = stgh_report_get_interval($dateStart, $dateEnd);

    $voted = array();

    /** @noinspection SqlResolve **/
    $query = "SELECT * from {$eventTable} WHERE {$whereDateInterval} ORDER BY ticket_id, created_at ASC";

    if ($result = $mysqli->query($query,MYSQLI_USE_RESULT)) {

        while ($row = $result->fetch_assoc()) {

            if(!isset($ticketId) || $row['ticket_id'] != $ticketId) {
                $ticketStats = array();
            }

            $ticketId = $row['ticket_id'];
            $ticketStats['ticket_category'] = $row['ticket_category'];

            $ticketStats['change_assignees'][$row['ticket_assignee']] = true;

            switch ($category>0) {
                case true:
                    switch ($row['event_type']) {
                        case 'reply_vote':
                            if($ticketStats['ticket_category'] == $category) {

                                foreach ($ticketStats['change_assignees'] as $assignee => $trash) {
                                    switch ($row['vote']) {
                                        case "1":
                                            $voted[$assignee]['theneutral'] = (isset($voted[$assignee]['theneutral'])) ? $voted[$assignee]['theneutral'] + 1 : 1;
                                            break;
                                        case "2":
                                            $voted[$assignee]['thegood'] = (isset($voted[$assignee]['thegood'])) ? $voted[$assignee]['thegood'] + 1 : 1;
                                            break;
                                        case "0":
                                            $voted[$assignee]['thebad'] = (isset($voted[$assignee]['thebad'])) ? $voted[$assignee]['thebad'] + 1 : 1;
                                            break;
                                    }
                                }

                            }
                            continue;
                        default:
                            continue;
                    }
                    break;
                default:
                    switch ($row['event_type']) {
                        case 'reply_vote':
                            foreach ($ticketStats['change_assignees'] as $assignee => $trash) {
                                switch ($row['vote']) {
                                    case "1":
                                        $voted[$assignee]['theneutral'] = (isset($voted[$assignee]['theneutral'])) ? $voted[$assignee]['theneutral'] + 1 : 1;
                                        break;
                                    case "2":
                                        $voted[$assignee]['thegood'] = (isset($voted[$assignee]['thegood'])) ? $voted[$assignee]['thegood'] + 1 : 1;
                                        break;
                                    case "0":
                                        $voted[$assignee]['thebad'] = (isset($voted[$assignee]['thebad'])) ? $voted[$assignee]['thebad'] + 1 : 1;
                                        break;
                                }
                            }
                            continue;
                        default:
                            continue;
                    }
            }
        }
        $result->free();
    }


    return $voted;

}