<?php
/**
 * Report class
 *
 * @link       https://mycatchers.com
 * @since      1.0.0
 *
 * @package    Catchers_Helpdesk_Addon5
 * @subpackage Catchers_Helpdesk_Addon5/includes
 */

use StgHelpdesk\Ticket\Stg_Helpdesk_TicketComments;

/**
 * Report class
 *
 *
 * @package    Catchers_Helpdesk_Addon5
 * @subpackage Catchers_Helpdesk_Addon5/includes
 * @author     mycatchers <support@mycatchers.com>
 */
class Catchers_Helpdesk_Addon5_Report  {


    public static function haveTasks(){
        global $wpdb;

        $tasksTable = $wpdb->prefix . STG_CATCHERS_HELPDESK_ADDON5_TASKS_TABLENAME;

        $checkTables = $wpdb->get_results("SHOW TABLES LIKE '{$tasksTable}';");

        if(empty($checkTables)) {
            return 0;
        }

        /** @noinspection SqlResolve */
        $query = "SELECT count(ticket_id) as cnt FROM {$tasksTable}";
        $result = $wpdb->get_var($query);
        return $result;
    }

    public static function deleteTables()
    {
        global $wpdb;

        $tableName = $wpdb->prefix . STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;
        $wpdb->query( "DROP TABLE IF EXISTS ".$tableName );

        $tableName = $wpdb->prefix . STG_CATCHERS_HELPDESK_ADDON5_TASKS_TABLENAME;
        $wpdb->query( "DROP TABLE IF EXISTS ".$tableName );

        $tableName = $wpdb->prefix . STG_CATCHERS_HELPDESK_ADDON5_UTABLENAME;
        $wpdb->query( "DROP TABLE IF EXISTS ".$tableName );
    }

	public static function deltaTables(){
        global $wpdb;

        $tableName = $wpdb->prefix . STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;
        $utableName = $wpdb->prefix . STG_CATCHERS_HELPDESK_ADDON5_UTABLENAME;

        $charsetCollate = '';

        if ( ! empty( $wpdb->charset ) ) {
            $charsetCollate = "DEFAULT CHARACTER SET {$wpdb->charset}";
        }
        if ( ! empty( $wpdb->collate ) ) {
            $charsetCollate .= " COLLATE {$wpdb->collate}";
        }

        $tableSql = "CREATE TABLE IF NOT EXISTS {$tableName} (
            id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
            created_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
            event_type VARCHAR(50) NOT NULL,
            ticket_channel VARCHAR(50) DEFAULT NULL,
            ticket_category TEXT DEFAULT NULL,
            ticket_assignee TEXT DEFAULT NULL,
            ticket_status TEXT DEFAULT NULL,
            ticket_created_at TIMESTAMP,
            ticket_id bigint(20) unsigned DEFAULT NULL,
            reply_id bigint(20) unsigned DEFAULT NULL,
            user_id bigint(20) unsigned DEFAULT NULL,
            vote int DEFAULT NULL,
            UNIQUE KEY id (id),
            KEY ticket_id (ticket_id),
            KEY user_id (user_id),
            KEY created_at (created_at)
            )
            ENGINE=InnoDB {$charsetCollate};";

        $wpdb->query($tableSql);

        $checkTables = $wpdb->get_results("SHOW TABLES LIKE '{$tableName}';");

        if(empty($checkTables)) {
            echo __('Reports table not created!',STG_HELPDESK_TEXT_DOMAIN_NAME);
            exit;
        }


        $tasksExists = $wpdb->get_var("SHOW TABLES LIKE '$utableName'") == $utableName;

        if($tasksExists === false) {
            /** @noinspection SqlResolve */
            $tableSql = "CREATE TABLE IF NOT EXISTS {$utableName} (
            ts TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,                         
            UNIQUE KEY contact_row (created_at, contact_id)
            )
            ENGINE=InnoDB {$charsetCollate} SELECT u.ID as contact_id, u.user_registered as created_at FROM {$wpdb->users} u LEFT JOIN {$wpdb->usermeta} m ON u.ID = m.user_id WHERE m.meta_key='wp_capabilities' and meta_value like '%\"stgh_client\"%';";

            $wpdb->query($tableSql);

            $checkTables = $wpdb->get_results("SHOW TABLES LIKE '{$tableName}';");

            if(empty($checkTables)) {
                echo __('Reports table not created!',STG_HELPDESK_TEXT_DOMAIN_NAME);
                exit;
            }
        }else{
            /** @noinspection SqlResolve */
            $tableSql = "INSERT IGNORE INTO {$utableName} (contact_id,created_at)
            SELECT u.ID as contact_id, u.user_registered as created_at FROM {$wpdb->users} u LEFT JOIN {$wpdb->usermeta} m ON u.ID = m.user_id WHERE m.meta_key='wp_capabilities' and meta_value like '%\"stgh_client\"%';";

            $wpdb->query($tableSql);
        }


        $tableName = $wpdb->prefix . STG_CATCHERS_HELPDESK_ADDON5_TASKS_TABLENAME;

        $charsetCollate = '';

        if ( ! empty( $wpdb->charset ) ) {
            $charsetCollate = "DEFAULT CHARACTER SET {$wpdb->charset}";
        }
        if ( ! empty( $wpdb->collate ) ) {
            $charsetCollate .= " COLLATE {$wpdb->collate}";
        }


        $tasksExists = $wpdb->get_var("SHOW TABLES LIKE '$tableName'") == $tableName;


        if($tasksExists === false) {
            /** @noinspection SqlResolve */
            $tableSql = "CREATE TABLE IF NOT EXISTS {$tableName} (
            status int(1) DEFAULT 0,
            created_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,            
            KEY created_at (created_at),
            KEY status (status)
            )
            ENGINE=InnoDB {$charsetCollate} SELECT ID as ticket_id FROM {$wpdb->posts} WHERE post_type ='" . STG_HELPDESK_POST_TYPE . "';";

            $wpdb->query($tableSql);

            $checkTables = $wpdb->get_results("SHOW TABLES LIKE '{$tableName}';");

            if(empty($checkTables)) {
                echo __('Reports tasks table not created!',STG_HELPDESK_TEXT_DOMAIN_NAME);
                exit;
            }


        }else{
            $lastTicketId =self::getLastTicketId();

            $notIncluded = ($lastTicketId > 0)? "and ID > $lastTicketId":"";

            /** @noinspection SqlResolve */
            $tableSql = "INSERT IGNORE INTO {$tableName} (ticket_id)
            SELECT ID as ticket_id FROM {$wpdb->posts} WHERE post_type ='" . STG_HELPDESK_POST_TYPE . "' ".$notIncluded.";";

            $wpdb->query($tableSql);
        }

    }

    public static function clearZombyTasks($time){
        global $wpdb;

        $taskTable = $wpdb->prefix . STG_CATCHERS_HELPDESK_ADDON5_TASKS_TABLENAME;

        $query = "UPDATE {$taskTable} SET status = 0 WHERE status=1 and {$time} - UNIX_TIMESTAMP(created_at) > 60";
        $result = $wpdb->query($query);

        return $result;
    }

    public static function getLastTicketId(){
        global $wpdb;

        $eventTable = $wpdb->prefix . STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;

        /** @noinspection SqlResolve */
        $query = "SELECT ticket_id FROM {$eventTable} ORDER BY created_at DESC LIMIT 1";
        $result = $wpdb->get_var($query);
        return $result;
    }

    public static function occupyTicket(){
        global $wpdb;

        $taskTable = $wpdb->prefix . STG_CATCHERS_HELPDESK_ADDON5_TASKS_TABLENAME;

        $affRow = $inc = $ticketId = 0;

        while($affRow == 0 && $inc < 10 && !is_null($ticketId)) {
            /** @noinspection SqlResolve */
            $query = "SELECT ticket_id FROM {$taskTable} WHERE status=0 ORDER BY created_at ASC LIMIT 1";
            $ticketId = $wpdb->get_var($query);

            if(!is_null($ticketId))
            {
                $query = "UPDATE {$taskTable} SET status = 1 WHERE ticket_id = {$ticketId} LIMIT 1";
                $affRow = $wpdb->query($query);
            }

            $inc++;

        }

        if($ticketId > 0)
        {
            return $ticketId;
        }else{
            throw new Exception('Can\'t occupy ticket from tasks!');
        }
    }

    public static function addTicketEvents($currTicketId){

            $currTicket = get_post($currTicketId);
            $currTicketMeta = get_post_meta($currTicketId);
            $currTicketComments = Stg_Helpdesk_TicketComments::instance($currTicketId)->get(array ( 'extra_params' => array ( 'post' => $currTicketId)), 'ASC');

            $ticketRow = self::getTicketRow($currTicket,$currTicketMeta);

            self::addNewTicketRow($ticketRow);

            if($currTicket->post_status == 'stgh_closed') {
                self::addCloseTicketRow($ticketRow,$currTicket);
            }

            $tickedAssignees = array();

            foreach($currTicketComments as $comment)
            {
                $commentMeta = get_post_meta($comment->ID);

                if(isset($commentMeta['_post_status_override_ticket_event_new']))
                {
                    $ticketStatus = self::addChangeStatusRow($ticketRow, $comment, $commentMeta);
                    if($ticketStatus !== null)
                        $ticketRow['ticket_status'] = $ticketStatus;
                }

                if(isset($commentMeta['_stgh_assignee_ticket_event_new']))
                {
                    $tickedAssignees[$commentMeta['_stgh_assignee_ticket_event_new'][0]] = true;

                    $ticketAssignee = self::addChangeAssigneeRow($ticketRow, $comment, $commentMeta);
                    if($ticketAssignee !== null)
                        $ticketRow['ticket_assignee'] = $ticketAssignee;
                }

                if(isset($commentMeta['_stgh_category_ticket_event_new']))
                {
                    $ticketCategory = self::addChangeCategoryRow($ticketRow, $comment, $commentMeta);
                    if($ticketCategory !== null)
                        $ticketRow['ticket_category'] = $ticketCategory;
                }

                if(!empty($comment->post_content)) {
                    self::addTicketReplyRow($ticketRow, $comment, $commentMeta);
                }


                if(isset($commentMeta['_stgh_crm_contact_value_ticket_event_new']))
                {
                    self::addChangeContactRow($ticketRow, $comment, $commentMeta);
                }

                if(isset($commentMeta['stgh_reply_vote']))
                {
                    self::addTicketVoteRow($ticketRow, $comment, $commentMeta, array_keys($tickedAssignees));
                }

            }
            self::removeTicketFromTasks($currTicketId);
    }


    static function removeTicketFromTasks($currTicketId){
        global $wpdb;

        $tasksTable = $wpdb->prefix . STG_CATCHERS_HELPDESK_ADDON5_TASKS_TABLENAME;
        $query = "DELETE FROM {$tasksTable} WHERE ticket_id={$currTicketId};";
        $result = $wpdb->query($query);
        return $result;
    }

    static function addEventRowToTable($eventRow){
        global $wpdb;

        $eventTable = $wpdb->prefix . STG_CATCHERS_HELPDESK_ADDON5_TABLENAME;

        $wpdb->insert($eventTable,$eventRow);
        $result = $wpdb->insert_id;
        return $result;
    }

    static function addUEventRowToTable($eventRow){
        global $wpdb;

        $eventTable = $wpdb->prefix . STG_CATCHERS_HELPDESK_ADDON5_UTABLENAME;

        $wpdb->insert($eventTable,$eventRow);
        $result = $wpdb->insert_id;
        return $result;
    }


    static function getTicketRow($currTicket, $currTicketMeta){

        $ticketChannel = (isset($currTicketMeta['_stgh_type_source']))? $currTicketMeta['_stgh_type_source'][0]:null;

        $agentTicket = (isset($currTicketMeta['_stgh_ticket_author_agent']))? $currTicketMeta['_stgh_ticket_author_agent'][0]:null;

        if($agentTicket == 1){
            $ticketChannel = (empty($ticketChannel))? __('agent', STG_HELPDESK_TEXT_DOMAIN_NAME):$ticketChannel;
        }


        $eventRow = array(
            'created_at' => $currTicket->post_date,
            'event_type' => null,
            'ticket_channel' => $ticketChannel,
            'ticket_category' => null,
            'ticket_status' => 'stgh_new',
            'ticket_assignee' => null,
            'ticket_created_at' => $currTicket->post_date,
            'ticket_id' => $currTicket->ID,
            'reply_id' => null,
            'user_id' => $currTicket->post_author,
            'vote' => null
        );

        return $eventRow;
    }

    /**
     * @param $currTicket WP_Post
     * @param $source
     * @return array
     */
    static function getTicketPublicRow($currTicket, $source){
        $eventRow = array(
            'created_at' => $currTicket->post_modified,
            'event_type' => null,
            'ticket_channel' => $source,
            'ticket_category' => null,
            'ticket_assignee' => null,
            'ticket_status' => $currTicket->post_status,
            'ticket_created_at' => $currTicket->post_date,
            'ticket_id' => $currTicket->ID,
            'reply_id' => null,
            'user_id' => $currTicket->post_author,
            'vote' => null
        );

        return $eventRow;
    }

    static function addCloseTicketRow($ticketRow,$currTicket){
        $ticketRow['event_type'] = 'change_status';
        $ticketRow['ticket_status'] = 'stgh_closed';
        $ticketRow['created_at'] = $currTicket->post_modified;
        self::addEventRowToTable($ticketRow);
    }

    static function addNewTicketRow($ticketRow){
        $ticketRow['event_type'] = 'new_ticket';
        self::addEventRowToTable($ticketRow);


        $ticketRow['event_type'] = 'change_status';
        $ticketRow['ticket_status'] = 'stgh_new';
        self::addEventRowToTable($ticketRow);
    }

    static function addTicketReplyRow($ticketRow, $comment, $commentMeta){

        $agentReply = isset($commentMeta['_stgh_type_color']) && $commentMeta['_stgh_type_color'][0] == 'agent';

        if($agentReply)
        {
            $ticketRow['event_type'] = 'agent_reply';
        }else{
            $ticketRow['event_type'] = 'client_reply';
        }

        $ticketRow['created_at'] = $comment->post_date;
        $ticketRow['reply_id'] = $comment->ID;
        $ticketRow['user_id'] =  $comment->post_author;

        self::addEventRowToTable($ticketRow);
    }

    static function addChangeAssigneeRow($ticketRow, $comment, $commentMeta){

        $ticketAssignee = (isset($commentMeta['_stgh_assignee_ticket_event_new']))? $commentMeta['_stgh_assignee_ticket_event_new'][0]:null;

        if($ticketAssignee !== null) {

            $ticketRow['event_type'] = 'change_assignee';
            $ticketRow['user_id'] = $ticketAssignee;
            $ticketRow['ticket_assignee'] = $ticketAssignee;
            $ticketRow['reply_id'] =  $comment->ID;
            $ticketRow['created_at'] =  $comment->post_date;

            self::addEventRowToTable($ticketRow);
        }

        return $ticketAssignee;
    }

    static function addChangeContactRow($ticketRow, $comment, $commentMeta){

        $ticketContact = (isset($commentMeta['_stgh_crm_contact_value_ticket_event_new']))? $commentMeta['_stgh_crm_contact_value_ticket_event_new'][0]:null;

        if($ticketContact !== null) {

            $ticketRow['event_type'] = 'change_contact';
            $ticketRow['reply_id'] =  $comment->ID;
            $ticketRow['created_at'] =  $comment->post_date;
            $ticketRow['user_id'] =  $ticketContact;

            self::addEventRowToTable($ticketRow);
        }
    }

    static function addChangeCategoryRow($ticketRow, $comment, $commentMeta){

        $ticketCategory = (isset($commentMeta['_stgh_category_ticket_event_new']))? $commentMeta['_stgh_category_ticket_event_new'][0]:null;

        if($ticketCategory !== null) {

            $ticketRow['event_type'] = 'change_category';
            $ticketRow['ticket_category'] = $ticketCategory;
            $ticketRow['reply_id'] =  $comment->ID;
            $ticketRow['created_at'] =  $comment->post_date;
            $ticketRow['user_id'] =  $comment->post_author;


            self::addEventRowToTable($ticketRow);
        }

        return $ticketCategory;
    }

    static function addChangeStatusRow($ticketRow, $comment, $commentMeta){

        $ticketStatus = (isset($commentMeta['_post_status_override_ticket_event_new']))? $commentMeta['_post_status_override_ticket_event_new'][0]:null;

        if($ticketStatus !== null) {

            $ticketRow['event_type'] = 'change_status';
            $ticketRow['ticket_status'] = $ticketStatus;
            $ticketRow['reply_id'] =  $comment->ID;
            $ticketRow['created_at'] =  $comment->post_date;
            $ticketRow['user_id'] =  $comment->post_author;


            self::addEventRowToTable($ticketRow);
        }

        return $ticketStatus;
    }

    static function addTicketVoteRow($ticketRow, $comment, $commentMeta, $tickedAssignees){

        $replyVote = (isset($commentMeta['stgh_reply_vote']))? $commentMeta['stgh_reply_vote'][0]:null;

        if(!empty($tickedAssignees) && $replyVote !== null) {

            $ticketRow['vote'] = $replyVote;
            $ticketRow['event_type'] = 'reply_vote';
            $ticketRow['reply_id'] =  $comment->ID;
            $ticketRow['created_at'] =  $comment->post_date;


            if(count($tickedAssignees) == 1) {
                $ticketRow['user_id'] =  $tickedAssignees[0];

                self::addEventRowToTable($ticketRow);
            }else{
                foreach($tickedAssignees as $tickedAssignee)
                {
                    $ticketRow['user_id'] =  $tickedAssignee;
                    self::addEventRowToTable($ticketRow);
                }
            }
        }
    }
}
