<?php
if (!function_exists('stg_wc_custom_orders_fields')) {
    function stg_wc_custom_orders_fields($actions, $order)
    {

        $actions['stg_order_get_help_'.stg_wc_get_order_id($order)] = array(
            'url' => '#',
            'name' => __('Open Ticket',STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME)
        );

        return $actions;
    }
}

if (!function_exists('stg_wc_get_order_id')) {
    function stg_wc_get_order_id($order)
    {
        if(method_exists($order,'get_id'))
        {
            return $order->get_id();
        }
        else
            return $order->id;
    }
}

if (!function_exists('stg_wc_get_currency')) {
    function stg_wc_get_currency($order)
    {
        if(method_exists($order,'get_currency'))
        {
            return $order->get_currency();
        }
        else
            return $order->order_currency;
    }
}

if (!function_exists('stg_wc_get_order_date')) {
    function stg_wc_get_order_date($order)
    {
        if(method_exists($order,'get_date_created'))
        {
            return $order->get_date_created()->date_i18n('Y-m-d H:m:s');
        }
        else
            return $order->order_date;
    }
}


if (!function_exists('stg_wc_get_woocommerce_currency_symbol')) {
    function stg_wc_get_woocommerce_currency_symbol()
    {
        return '';
        //return get_woocommerce_currency_symbol($abb);
    }
}


if (!function_exists('stg_wc_get_customer_last_order')) {
    function stg_wc_get_customer_last_order($customerId)
    {
        return wc_get_customer_last_order($customerId);
    }
}

if (!function_exists('stg_wc_get_customer_orders')) {
    function stg_wc_get_customer_orders($customerId, $limit = false)
    {
        $args = array(
            'status'   => array_keys( wc_get_order_statuses() ),
            'type'     => wc_get_order_types( 'view-orders' ),
            'customer' => $customerId,
            'limit'    => 5,
            'orderby'  => 'date',
            'order'    => 'DESC',
            'return'   => 'objects',
            'paginate' => false,
        );

        $orders = wc_get_orders($args);

        if($limit !== false){
            $orders = array_slice($orders,0,$limit,true);
        }

        return $orders;
    }
}

if (!function_exists('stg_wc_get_customer_lifetime')) {
    function stg_wc_get_customer_lifetime($customerId)
    {

        $customer_orders = stg_wc_get_customer_orders($customerId);

        $lifetime = 0;

        /* @var $order WC_Order */
        foreach($customer_orders as $order)
        {
            $orderTotal = $order->get_total();
            $lifetime += $orderTotal;
        }

        return wc_price($lifetime);
    }
}

if (!function_exists('stg_wc_get_ticket_items')) {
    function stg_wc_get_ticket_items($ticketId, $limit = false)
    {
        $result = array();
        $productIds = get_post_meta($ticketId,'stg_wc_ticket_products', false);

        if(!empty($productIds)){

            if($limit !== false){
                $productIds = array_slice($productIds,0,$limit);
            }

            foreach($productIds as $productId)
            {
                $result[] = wc_get_product( $productId );
            }
        }

        return $result;
    }
}

if (!function_exists('stg_wc_get_order_products')) {
    function stg_wc_get_order_products($orderId, $order = false)
    {
        if($order === false){
            $order = wc_get_order($orderId);
        }

        $items = $order->get_items();
        return $items;
    }
}

if (!function_exists('stg_wc_get_order_total_cost')) {
    function stg_wc_get_order_total_cost($orderId,$order = false)
    {
        if($order === false){
            $order = wc_get_order($orderId);
        }
        $items = $order->get_items();
        $result = 0;
        foreach($items as $item){
            $result+=$item['line_total'];
        }
        return wc_price($result);
    }
}

if (!function_exists('stg_wc_get_ticket_order')) {
    function stg_wc_get_ticket_order($ticketId,$currentContactUserId=false)
    {
        $orderId = get_post_meta($ticketId,'stg_wc_ticket_order', true);

        if(!empty($orderId)){
            $order = wc_get_order($orderId);;
        }
        else{

            $orders = stg_wc_get_customer_orders($currentContactUserId);
            $order = isset($orders[0])? $orders[0]: false;

        }

        return $order;
    }
}

if (!function_exists('stg_wc_get_user_form')) {
    function stg_wc_get_user_form()
    {

        $formId = sanitize_text_field($_REQUEST['form']);
        $orderId = sanitize_text_field($_REQUEST['orderId']);

        if(!empty($formId))
        {
            $result = \StgHelpdesk\Helpers\Stg_Helper_Template::getTemplate('stg-ticket-form-by-id', array("formId" => $formId), false);
            if(strpos($result,STG_HELPDESK_WC_PRODUCT_PLACEHOLDER) === false)
            {
                $result = \StgHelpdesk\Helpers\Stg_Helper_Template::getTemplate('stg-ticket-form', array(), false);
            }
        }else{
            $defaultForm= \StgHelpdesk\Helpers\Stg_Helper_Custom_Forms::getDefaultForm();

            if($defaultForm)
            {
                $result = \StgHelpdesk\Helpers\Stg_Helper_Template::getTemplate('stg-ticket-form-by-id', array("formId" => $defaultForm->term_id), false);
                if(strpos($result,STG_HELPDESK_WC_PRODUCT_PLACEHOLDER) === false)
                {
                    $result = \StgHelpdesk\Helpers\Stg_Helper_Template::getTemplate('stg-ticket-form', array(), false);
                }
            }else{
                $result = \StgHelpdesk\Helpers\Stg_Helper_Template::getTemplate('stg-ticket-form', array(), false);
            }
        }



	    if($orderId !== 'NOORDER') {
		    $result = stg_wc_add_blocks_to_form( $result, $orderId );
	    }

        echo $result;
        die();
    }
}

if (!function_exists('stgh_ajax_get_product_search')) {
    function stgh_ajax_get_product_search()
    {
        $searchQ = sanitize_text_field($_REQUEST['q']);

        $productsIds = $_REQUEST['idsa'];
        $productsIds = array_map( 'esc_attr', $productsIds);

        $result = stgh_get_product_search($searchQ, $productsIds);

        header('Content-Type: application/x-javascript');
        echo json_encode($result);
        exit;
    }
}

if (!function_exists('stgh_get_product_search')) {
    function stgh_get_product_search($q, $excludeIds = array(), $limit = 5)
    {

        $result = array();

        $query = new \WP_Query(
            array(
                'post_type' => 'product',
                'post_status' => 'any',
                'orderby' => 'post_date',
                'order' => 'DESC',
                'posts_per_page' => $limit,
                'post__not_in' => $excludeIds,
                's' => $q
            )
        );

        if ($query->have_posts()) {
            foreach($query->posts as $post){
                $result[] = array('id'=> $post->ID, 'full_name' => "# ".$post->ID." ".$post->post_title,'text' => $post->post_content);
            }
        }

        wp_reset_query();

        return $result;
    }
}

if (!function_exists('stg_wc_save_ticket_meta')) {
	function stg_wc_save_ticket_meta($postId)
	{

		$stgSaveTicket = (isset($_REQUEST['stg_saveTicket']))? sanitize_text_field($_REQUEST['stg_saveTicket']):null;
		$orderId = (isset($_REQUEST['stg_ticket_order_id']))? sanitize_text_field($_REQUEST['stg_ticket_order_id']):null;
		$stgh_agent_ticket= (isset($_REQUEST['stgh_agent_ticket']))? sanitize_text_field($_REQUEST['stgh_agent_ticket']):null;


		$productsIds = (isset($_REQUEST['stgh_wc_order_products']))? $_REQUEST['stgh_wc_order_products']:array();

		$productsIds = array_map( 'esc_attr', $productsIds);

		if ($stgSaveTicket == 1) {
			add_post_meta($postId, 'stg_wc_ticket_order', $orderId, true);
			foreach($productsIds as $productId)
				add_post_meta($postId, 'stg_wc_ticket_products', $productId, false);
		}else {
			if ( $stgh_agent_ticket == 1 ) {
				add_post_meta( $postId, 'stg_wc_ticket_order', $orderId, true );
			}
		}
	}
}

if (!function_exists('stg_wc_add_blocks_to_form')) {
    function stg_wc_add_blocks_to_form($result,$orderId)
    {
        $addOrderBlock = '<input class="stgh_standart_field" name="stg_ticket_order_id" type="hidden" value="'.$orderId.'"/>';

        $needle = STG_HELPDESK_WC_PRODUCT_PLACEHOLDER;
        $addProductBlock = stg_wc_get_form_product_block($orderId);

        if(strpos($result,$needle) !== false)
        {
            $result = str_replace($needle,$addProductBlock.$addOrderBlock,$result);
        }

        return $result;
    }
}

if (!function_exists('stg_wc_get_form_product_block')) {
    function stg_wc_get_form_product_block($orderId)
    {
        $products = stg_wc_get_order_products($orderId);

        $optionsArr = array();
        foreach($products as $product)
        {
            $optionsArr[] = $product['product_id'].":".$product['name'];
        }

        $options = "";
        $required = "";

        foreach($optionsArr as $option)
        {
            if(!empty($option)) {
                @list($currValue,$text) = explode(":",$option);
                if(!$text) {
                    $currValue = trim($currValue);
                    $text = $currValue;

                }

                $options .= "<input type='checkbox' name='stgh_wc_order_products[]' {$required} value='{$currValue}' id='stgh_wc_pro{$currValue}'/><label for='stgh_wc_pro{$currValue}'>{$text}</label><br/>";

            }

        }


        return "<strong><label>".__('Order Items', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME).":</label></strong><div id='stg_wc_products_block'>".$options."</div>";
    }
}

if (!function_exists('stg_wc_register_metaboxes')) {
    function stg_wc_register_metaboxes()
    {
        global $post_type, $post;

        if ($post_type == STG_HELPDESK_POST_TYPE or $post_type == STG_HELPDESK_COMMENTS_POST_TYPE) {

            $ticket = StgHelpdesk\Ticket\Stg_Helpdesk_Ticket::getInstance($post->ID);
            $contact = $ticket->getContact();

            if(!empty($contact))
            {
                $currentOrder = stg_wc_get_ticket_order($post->ID,$contact->ID);

                if($currentOrder) {
                    $url = get_admin_url(null, "post.php?post=".stg_wc_get_order_id($currentOrder)."&action=edit");

                    add_meta_box('stgh_ticket_wc_order', __('Order', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME).' <a href="'.$url.'" class="stgh-notunderline"># <span class="stgh-underline">'.stg_wc_get_order_id($currentOrder).'</span></a>', 'stg_wc_add_box',
                        STG_HELPDESK_POST_TYPE,
                        'side', 'high', array('template' => 'user_order'));
                }

                $ticketProducts = stg_wc_get_ticket_products($post->ID);

                if(count($ticketProducts) > 0) {
                    add_meta_box('stgh_ticket_wc_products', __('Products', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME), 'stg_wc_add_box',
                        STG_HELPDESK_POST_TYPE,
                        'side', 'high', array('template' => 'ticket_product'));
                }
            }

        }


        if ($post_type == 'shop_order') {

            add_meta_box('stgh_wc_order_tickets', __('Last tickets', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME), 'stg_wc_add_box',
                $post_type,
                'side', 'low', array('template' => 'order_tickets'));
        }

        if ($post_type == 'product') {
            add_meta_box('stgh_wc_product_tickets', __('Last tickets', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME), 'stg_wc_add_box',
                $post_type,
                'side', 'low', array('template' => 'product_tickets'));
        }
    }
}

if (!function_exists('stg_wc_get_ticket_products')) {
    function stg_wc_get_ticket_products($ticketId, $limit = false)
    {
        $result = array();

        $productIds = get_post_meta($ticketId,'stg_wc_products', true);

        if(!empty($productIds)){

            if($limit !== false){
                $productIds = array_slice($productIds,0,$limit);
            }

            foreach($productIds as $productId)
            {
                $result[] = wc_get_product( $productId );
            }
        }

        return $result;
    }
}



if (!function_exists('stg_wc_add_box')) {
    function stg_wc_add_box($post, $args)
    {
        if (!is_array($args) || !isset($args['args']['template'])) {
            _e('An error occurred while registering this metabox. Please contact the support.', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME);
        }

        $template = $args['args']['template'];
        $template = STG_HELPDESK_WC_SUPPORT_ROOT . "admin/metaboxes/$template.php";

        if (!file_exists($template)) {
            _e('An error occured while loading this metabox. Please contact the support.', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME);
        }

        include_once($template);
    }
}

if (!function_exists('stg_wc_add_product_placeholder')) {
    function stg_wc_add_product_placeholder()
    {
        echo STG_HELPDESK_WC_PRODUCT_PLACEHOLDER;

        $isRecaptcha = stg_recaptcha_enabled();

        if($isRecaptcha)
        {
            echo \StgHelpdesk\Helpers\Stg_Helper_Custom_Forms::getStandartFieldReCAPTCHA();
        }

    }
}

if (!function_exists('stg_wc_add_products_in_selector')) {
    function stg_wc_add_products_in_selector($result)
    {
        $result[2]['children'][] = array('text' => __('Order Items', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME), 'id' => STG_HELPDESK_WC_PRODUCT_PLACEHOLDER);
        $result[2]['children'][] = array('text' => __('Products', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME), 'id' => '[custom_field \'stg_wc_products\']','label' => __('Products', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME));
        return $result;
    }
}

if (!function_exists('stg_wc_remove_products_from_array')) {
    function stg_wc_remove_products_from_array($result)
    {
        foreach($result[2]['children'] as $key => $value)
        {
            if($value['id'] == STG_HELPDESK_WC_PRODUCT_PLACEHOLDER)
                unset($result[2]['children'][$key]);

            if($value['id'] == '[custom_field \'stg_wc_products\']')
                unset($result[2]['children'][$key]);
        }
        return $result;
    }
}

if (!function_exists('stg_wc_form_text_add_replace')) {
    function stg_wc_form_text_add_replace($result)
    {
        $result[] = array(
            'search'=> '[custom_field \'stg_wc_products\']',
            'replace'=> "<select id='stg_wc_products_field' name='stgh_custom_fields[stg_wc_products][]' class='stgh_custom_fields' multiple='multiple'></select>");

        return $result;
    }
}


if (!function_exists('stg_wc_get_order_tickets')) {
    function stg_wc_get_order_tickets($orderId, $limit)
    {

        $query = new \WP_Query(
            array(
                'post_type' => STG_HELPDESK_POST_TYPE,
                'post_status' => 'any',
                'orderby' => 'post_date',
                'order' => 'DESC',
                'meta_query' => array(
                    array(
                        'key' => 'stg_wc_ticket_order',
                        'value' => $orderId
                    )
                )
            )
        );
        $result = array();
        if ($query->have_posts()) {
            $posts = array_slice($query->posts,0,$limit);
            foreach($posts as $post){
                $url = get_admin_url(null, "post.php?post_type=".STG_HELPDESK_POST_TYPE."&post={$post->ID}&action=edit");
                $result[] = array('name'=>$post->post_title,'url'=>$url);
            }
        }
        wp_reset_query();
        return $result;
    }
}

if (!function_exists('stg_wc_get_product_tickets')) {
    function stg_wc_get_product_tickets($productId, $limit)
    {

        $query = new \WP_Query(
            array(
                'post_type' => STG_HELPDESK_POST_TYPE,
                'post_status' => 'any',
                'orderby' => 'post_date',
                'order' => 'DESC',
                'meta_query' => array(
                    'relation'  => 'OR',
                    array(
                        'key'       => 'stg_wc_products',
                        'value'     => '"'.$productId.'"',
                        'compare'   => 'like',
                    ),
                    array(
                        'key' => 'stg_wc_ticket_products',
                        'value' => $productId,
                        'compare' => '='
                    )
                )
            )
        );

        $result = array();
        if ($query->have_posts()) {
            $posts = array_slice($query->posts,0,$limit);
            foreach($posts as $post){
                $url = get_admin_url(null, "post.php?post_type=".STG_HELPDESK_POST_TYPE."&post={$post->ID}&action=edit");
                $result[] = array('name'=>$post->post_title,'url'=>$url);
            }
        }
        wp_reset_query();
        return $result;
    }
}

if (!function_exists('getSelectL10nArray')) {
    function getSelectL10nArray()
    {
        $result = array(
            'errorLoading' => __("The results could not be loaded.", STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
            'inputTooLong_n' => __("Please delete", STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
            'inputTooLong_nplus' => __("characters", STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
            'inputTooShort_n' => __("Please enter", STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
            'inputTooShort_nplus' => __("or more characters", STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
            'loadingMore' => __("Loading more results...", STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
            'maximumSelected_n' => __("You can only select", STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
            'maximumSelected_nplus' => __("items", STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
            'noResults' => __("No results found", STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
            'searching' => __("Searching...", STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME)
        );
        return $result;
    }
}

if (!function_exists('stg_wc_add_general_settings')) {
    function stg_wc_add_general_settings($settings)
    {
        if(\is_plugin_active('catchers-helpdesk-addon1/catchers-helpdesk-addon1.php'))
        {
            $contactForms = \StgHelpdesk\Helpers\Stg_Helper_Custom_Forms::getContactForms(true);
            $defaultForm = \StgHelpdesk\Helpers\Stg_Helper_Custom_Forms::getDefaultForm();

            $contactFormsPrep = array();
            foreach($contactForms as $contactForm)
            {
                if(strpos($contactForm->description,STG_HELPDESK_WC_PRODUCT_PLACEHOLDER) !== false || $defaultForm->term_id == $contactForm->term_id)
                {
                    $contactFormsPrep[$contactForm->term_id] = $contactForm->name;
                }
            }


            $ticketFormSelector = array(
                'name'     => __( 'Contact form', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
                'id'       => 'stg_wc_custom_form',
                'type'     => 'select',
                'default'  => $defaultForm->term_id,  // WC >= 2.0
                'desc'     => __( 'Choose custom contact form. Attention! This selector contains only form with order items line', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME ),
                'options' => $contactFormsPrep,
            );
        }


        $updated_settings = array(
            array(
                'title'     => __( 'Catchers Helpdesk section', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
                'id'       => 'catcher_helpdesk_options',
                'type'     => 'title',
            ),
            array(
                'name'     => __( 'Add capabilities', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
                'id'       => 'stg_wc_stgh_manager_to_shop',
                'type'     => 'checkbox',
                'std'      => false,  // WC < 2.0
                'default'  => false,  // WC >= 2.0
                'desc'     => __( 'Allow to Helpdesk Manager have Shop Manager capabilities', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME ),
            )
        );

        $sectionEnd = array(
            'id'       => 'catcher_helpdesk_options',
            'type'     => 'sectionend',
        );

        if(isset($ticketFormSelector) && is_array($ticketFormSelector))
            $updated_settings[] = $ticketFormSelector;

        $updated_settings[] = $sectionEnd;

        $settings = array_merge($settings,$updated_settings);

        return $settings;

    }
}

if (!function_exists('stg_wc_add_general_settings_save')) {
    function stg_wc_add_general_settings_save()
    {
        $option = get_option( 'stg_wc_stgh_manager_to_shop', 1 );
        switch($option){
            case 'yes':
                stg_wc_add_manager_capabilities();
                break;
            case 'no':
                stg_wc_remove_manager_capabilities();
                break;
        }
    }
}

if (!function_exists('stg_wc_get_capabilities_diff')) {
    function stg_wc_get_capabilities_diff()
    {

        return array('assign_product_terms' => true, 'assign_shop_coupon_terms' => true, 'assign_shop_order_terms' => true, 'assign_shop_webhook_terms' => true, 'delete_others_pages' => true, 'delete_others_posts' => true, 'delete_others_products' => true, 'delete_others_shop_coupons' => true, 'delete_others_shop_orders' => true, 'delete_others_shop_webhooks' => true, 'delete_pages' => true, 'delete_posts' => true, 'delete_private_pages' => true, 'delete_private_posts' => true, 'delete_private_products' => true, 'delete_private_shop_coupons' => true, 'delete_private_shop_orders' => true, 'delete_private_shop_webhooks' => true, 'delete_product' => true, 'delete_product_terms' => true, 'delete_products' => true, 'delete_published_pages' => true, 'delete_published_posts' => true, 'delete_published_products' => true, 'delete_published_shop_coupons' => true, 'delete_published_shop_orders' => true, 'delete_published_shop_webhooks' => true, 'delete_shop_coupon' => true, 'delete_shop_coupon_terms' => true, 'delete_shop_coupons' => true, 'delete_shop_order' => true, 'delete_shop_order_terms' => true, 'delete_shop_orders' => true, 'delete_shop_webhook' => true, 'delete_shop_webhook_terms' => true, 'delete_shop_webhooks' => true, 'edit_others_pages' => true, 'edit_others_posts' => true, 'edit_others_products' => true, 'edit_others_shop_coupons' => true, 'edit_others_shop_orders' => true, 'edit_others_shop_webhooks' => true, 'edit_pages' => true, 'edit_private_pages' => true, 'edit_private_posts' => true, 'edit_private_products' => true, 'edit_private_shop_coupons' => true, 'edit_private_shop_orders' => true, 'edit_private_shop_webhooks' => true, 'edit_product' => true, 'edit_product_terms' => true, 'edit_products' => true, 'edit_published_pages' => true, 'edit_published_posts' => true, 'edit_published_products' => true, 'edit_published_shop_coupons' => true, 'edit_published_shop_orders' => true, 'edit_published_shop_webhooks' => true, 'edit_shop_coupon' => true, 'edit_shop_coupon_terms' => true, 'edit_shop_coupons' => true, 'edit_shop_order' => true, 'edit_shop_order_terms' => true, 'edit_shop_orders' => true, 'edit_shop_webhook' => true, 'edit_shop_webhook_terms' => true, 'edit_shop_webhooks' => true, 'export' => true, 'import' => true, 'level_1' => true, 'level_2' => true, 'level_3' => true, 'level_4' => true, 'level_5' => true, 'level_6' => true, 'level_7' => true, 'level_8' => true, 'level_9' => true, 'manage_links' => true, 'manage_product_terms' => true, 'manage_shop_coupon_terms' => true, 'manage_shop_order_terms' => true, 'manage_shop_webhook_terms' => true, 'manage_woocommerce' => true, 'moderate_comments' => true, 'publish_pages' => true, 'publish_posts' => true, 'publish_products' => true, 'publish_shop_coupons' => true, 'publish_shop_orders' => true, 'publish_shop_webhooks' => true, 'read_private_pages' => true, 'read_private_posts' => true, 'read_private_products' => true, 'read_private_shop_coupons' => true, 'read_private_shop_orders' => true, 'read_private_shop_webhooks' => true, 'read_product' => true, 'read_shop_coupon' => true, 'read_shop_order' => true, 'read_shop_webhook' => true, 'unfiltered_html' => true, 'view_woocommerce_reports' => true);
    }
}

if (!function_exists('stg_wc_add_manager_capabilities')) {
    function stg_wc_add_manager_capabilities()
    {
        $stgh_manager = get_role('stgh_manager');
        $caps = stg_wc_get_capabilities_diff();

        foreach ($caps as $cap_key => $cap_value)
        {
            if($cap_value)
            {
                $stgh_manager->add_cap($cap_key);
            }
        }
    }
}

if (!function_exists('stg_wc_remove_manager_capabilities')) {
    function stg_wc_remove_manager_capabilities()
    {
        $stgh_manager = get_role('stgh_manager');
        $caps = stg_wc_get_capabilities_diff();

        foreach ($caps as $cap_key => $cap_value)
        {
            if($cap_value)
            {
                $stgh_manager->remove_cap($cap_key);
            }
        }
    }
}

if (!function_exists('stg_wc_add_order_list_box')) {
	function stg_wc_add_order_list_box()
	{
		$args = array(
			'class' => 'stgh-ticket-property',
			'name' => 'stg_ticket_order_id',
			'id' => 'stg_ticket_order_id',
			'cap' => 'edit_ticket',
			'please_select_text' => __('Choose order', STG_HELPDESK_TEXT_DOMAIN_NAME),
			'stgselect2' => true,
			'role' => 'stgh_manager',
			'disabled' => false
		);
		$options = array();

		echo "<span id='stgh-ticket-order-list-box'>".stgh_dropdown($args, $options)."</span>";
	}
}