<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://mycatchers.com
 * @since      1.0.0
 *
 * @package    Catchers_WC_Support
 * @subpackage Catchers_WC_Support/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Catchers_WC_Support
 * @subpackage Catchers_WC_Support/public
 * @author     Your Name <email@example.com>
 */
class Catchers_WC_Support_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Catchers_WC_Support_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Catchers_WC_Support_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style('stgh-select2', STG_HELPDESK_URL . 'css/vendor/stgselect2/stgselect2.min.css', null, '4.0.0', 'all');
		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/catchers-wc-support-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		global $post;

		$isWCmuaccount = get_the_ID() == get_option( 'woocommerce_myaccount_page_id' );

		$isCatchersForm = false;

		if( is_a( $post, 'WP_Post' )) {

			$hasForm = strpos($post->post_content,'stg-ticket-form');
			$hasTag =  has_shortcode( $post->post_content, STG_HELPDESK_SHORTCODE_TICKET_FORM);

			$isCatchersForm = $hasTag or $hasForm !== false;

		}


		if($isWCmuaccount or $isCatchersForm)
		{
			$isRecaptcha = stg_recaptcha_enabled();

			if($isRecaptcha)
			{
				$grecaptchaKey = stgh_get_option('recaptcha_key');
				$grecaptchaSKey = stgh_get_option('recaptcha_secret_key');
				$grecaptchaHL = stgh_get_option('recaptcha_hl');

				if($grecaptchaKey && $grecaptchaSKey) {
					wp_enqueue_script('grecaptcha', '//www.google.com/recaptcha/api.js?hl='.$grecaptchaHL, false, null);
				}

			}



			wp_enqueue_script('jquery-ui-draggable');
			wp_enqueue_script('jquery-ui-droppable');

			wp_enqueue_script('stgh-select2', STG_HELPDESK_URL . 'js/vendor/stgselect2.min.js', array('jquery'), '4.0.0', true);
			wp_localize_script('stgh-select2', 'stgh_select2_l10n', getSelectL10nArray());



		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/catchers-wc-support-public.js', array( 'jquery','stgh-select2' ), $this->version, false );

		$form = get_option( 'stg_wc_custom_form', false );

		wp_localize_script($this->plugin_name, 'stgh_public',
			array(
				'ajaxurl' => admin_url('admin-ajax.php'),
				'button_color' => stgh_get_option('helpcatcher_button_color'),
				'contactForm' => $form,
			));

		}

	}

}
